@@ -1,15 +1,13 @@
 # application.py: Application class to store properties of the application
 #                 being created
-# $Id: application.py,v 1.63 2007/04/02 14:12:13 agriggio Exp $
+# $Id: application.py,v 1.64 2007/08/07 12:13:44 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
-#from wxPython.wx import *
 import wx
 from widget_properties import *
-from tree import Tree, WidgetTree
 import common, math, misc, os, config
 import traceback, re
 
@@ -18,8 +16,8 @@ class FileDirDialog:
     Custom class which displays a FileDialog or a DirDialog, according to the
     value of the codegen_opt of its parent (instance of Application)
     """
-    def __init__(self, owner, parent, wildcard="All Files|*",
-                 file_message="Choose a file", dir_message=None, style=0):
+    def __init__(self, owner, parent, wildcard=_("All Files|*"),
+                 file_message=_("Choose a file"), dir_message=None, style=0):
         self.owner = owner
         self.prev_dir = config.preferences.codegen_path
         self.wildcard = wildcard
@@ -27,42 +25,17 @@ class FileDirDialog:
         self.dir_message = dir_message
         self.file_style = style
         self.dir_style = wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON
-##         self.file_dialog = wxFileDialog(parent, file_message, self.prev_dir,
-##                                         wildcard=wildcard, style=style)
-##         if dir_message is None: dir_message = file_message
-##         log_null = wxLogNull() # to prevent popup messages about lack of
-##                                # permissions to view the contents of
-##                                # some directories
-##         style = 0
-##         if misc.check_wx_version(2, 3, 3):
-##             style = wxDD_DEFAULT_STYLE|wxDD_NEW_DIR_BUTTON
-##         self.dir_dialog = wxDirDialog(parent, dir_message, style=style)
-##         del log_null
         self.parent = parent
-##         self.file_message = file_message
-##         self.style = style
         self.value = None
 
     def ShowModal(self):
         if self.owner.codegen_opt == 0:
-##             if self.prev_dir is not None:
-##                 self.file_dialog.SetDirectory(self.prev_dir)
-##             dialog = self.file_dialog
             self.value = misc.FileSelector(
                 self.file_message, self.prev_dir or "",
                 wildcard=self.wildcard, flags=self.file_style)
         else:
-##             if self.prev_dir is not None:
-##                 self.dir_dialog.SetPath(self.prev_dir)
-##             dialog = self.dir_dialog
             self.value = misc.DirSelector(
                 self.dir_message, self.prev_dir or "", style=self.dir_style)
-##         ok = dialog.ShowModal()
-##         if ok == wxID_OK:
-##             self.prev_dir = dialog.GetPath()
-##             if not os.path.isdir(self.prev_dir):
-##                 self.prev_dir = os.path.dirname(self.prev_dir)
-##         return ok
         if self.value:
             self.prev_dir = self.value
             if not os.path.isdir(self.prev_dir):
@@ -71,18 +44,9 @@ class FileDirDialog:
         return wx.ID_CANCEL
 
     def get_value(self):
-##         if self.owner.codegen_opt == 0: return self.file_dialog.GetPath()
-##         else: return self.dir_dialog.GetPath()
         return self.value
 
     def set_wildcard(self, wildcard):
-##         if wxPlatform == '__WXMSW__':
-##             self.file_dialog.SetWildcard(wildcard)
-##         else:
-##             # on GTK SetWildcard has no effect, so we recreate the dialog
-##             self.file_dialog = wxFileDialog(self.parent, self.file_message,
-##                                             wildcard=wildcard,
-##                                             style=self.style)
         self.wildcard = wildcard
         
 # end of class FileDirDialog
@@ -102,8 +66,15 @@ class Application(object):
             self.notebook.sizer = None
         self.notebook.SetAutoLayout(True)
         self.notebook.Hide()
-        #panel = wx.Panel(self.notebook, -1)
-        panel = wx.ScrolledWindow(self.notebook, -1, style=wx.TAB_TRAVERSAL)
+        panel = wx.ScrolledWindow(
+            self.notebook, -1, style=wx.TAB_TRAVERSAL|wx.FULL_REPAINT_ON_RESIZE)
+        lang_panels = {}
+        def mkp():
+            return wx.ScrolledWindow(
+                self.notebook, -1,
+                style=wx.TAB_TRAVERSAL|wx.FULL_REPAINT_ON_RESIZE)
+        lang_panels['C++'] = mkp()
+        lang_panels['python'] = mkp()
         self.name = "app" # name of the wxApp instance to generate
         self.__saved = True # if True, there are no changes to save
         self.__filename = None # name of the output xml file
@@ -114,11 +85,25 @@ class Application(object):
             try: opt = int(value)
             except ValueError: pass
             else: self.codegen_opt = opt
+        self.indent_mode = 1
+        self.indent_amount = 4
+        def set_indent_mode(value):
+            try: opt = int(value)
+            except ValueError: pass
+            else: self.indent_mode = opt
+        def set_indent_amount(value):
+            try: opt = int(value)
+            except ValueError: pass
+            else: self.indent_amount = opt
+        self.source_ext = 'cpp'
+        self.header_ext = 'h'
+        def set_source_ext(value): self.source_ext = value
+        def set_header_ext(value): self.header_ext = value
         self.output_path = ""
         self.language = 'python' # output language
         def get_output_path(): return os.path.expanduser(self.output_path)
         def set_output_path(value): self.output_path = value
-	self.is_template = False
+        self.is_template = False
         self.use_gettext = False
         def set_use_gettext(value): self.use_gettext = bool(int(value))
         self.for_version = wx.VERSION_STRING[:3]
@@ -128,29 +113,46 @@ class Application(object):
             'name': (lambda : self.name, self.set_name),
             'class': (lambda : self.klass, self.set_klass), 
             'code_generation': (lambda : self.codegen_opt, set_codegen_opt),
+            'indent_mode': (lambda : self.indent_mode, set_indent_mode),
+            'indent_amount': (lambda : self.indent_amount, set_indent_amount),
+            'source_ext' : (lambda : self.source_ext, set_source_ext),
+            'header_ext' : (lambda : self.header_ext, set_header_ext),
             'output_path': (get_output_path, set_output_path),
             'language': (self.get_language, self.set_language),
             'encoding': (self.get_encoding, self.set_encoding),
             'use_gettext': (lambda : self.use_gettext, set_use_gettext),
             'for_version': (lambda : self.for_version, set_for_version),
             }
-        self.use_gettext_prop = CheckBoxProperty(self, "use_gettext", panel,
-                                                 "Enable gettext support")
-        TOP_WIN_ID = wx.NewId()
-        self.top_win_prop = wx.Choice(panel, TOP_WIN_ID, choices=[],
-                                     size=(1, -1))
-        self.top_window = '' # name of the top window of the generated app
-
         self.name_prop = TextProperty(self, "name", panel, True)
         self.klass_prop = TextProperty(self, "class", panel, True)
 
         self.encoding = self._get_default_encoding()
         self.encoding_prop = TextProperty(self, 'encoding', panel)
+
+        self.use_gettext_prop = CheckBoxProperty(self, "use_gettext", panel,
+                                                 _("Enable gettext support"))
+        TOP_WIN_ID = wx.NewId()
+        self.top_win_prop = wx.Choice(panel, TOP_WIN_ID, choices=[],
+                                     size=(1, -1))
+        self.top_window = '' # name of the top window of the generated app
+
         
         self.codegen_prop = RadioProperty(self, "code_generation", panel,
-                                          ["Single file", "Separate file for" \
-                                           " each class"])
-
+                                          [_("Single file"),
+                                           _("Separate file for" \
+                                           " each class")],
+                                          label=_("Code Generation"))
+        self.indent_mode_prop = RadioProperty(self, "indent_mode",
+                                              lang_panels['C++'],
+                                              [_("Tabs"), _("Spaces")],
+                                              columns=2,
+                                              label=_("Indentation mode"))
+        self.indent_amount_prop = SpinProperty(self, 'indent_amount',
+                                               lang_panels['C++'], r=(1, 100))
+        self.source_ext_prop = TextProperty(self, 'source_ext',
+                                            lang_panels['C++'])
+        self.header_ext_prop = TextProperty(self, 'header_ext',
+                                            lang_panels['C++'])
         ext = getattr(common.code_writers.get('python'),
                       'default_extensions', [])
         wildcard = []
@@ -158,14 +160,10 @@ class Application(object):
             wildcard.append('%s files (*.%s)|*.%s' % ('Python', e, e))
         wildcard.append('All files|*')
         dialog = FileDirDialog(self, panel, '|'.join(wildcard),
-                               "Select output file", "Select output directory",
+                               _("Select output file"), _("Select output directory"),
                                wx.SAVE|wx.OVERWRITE_PROMPT)
 
-##         # this "columns-stuff" is to fix a bug (of at least wxGTK)
         _writers = common.code_writers.keys()
-##         if not len(_writers) % 3: columns = 3
-##         elif not len(_writers) % 2: columns = 2
-##         else: columns = 1
         columns = 3
 
         self.codewriters_prop = RadioProperty(self, "language", panel,
@@ -175,15 +173,15 @@ class Application(object):
 
         self.for_version_prop = RadioProperty(self, "for_version", panel,
                                               ['2.4', '2.6', '2.8'], columns=3,
-                                              label="wxWidgets compatibility")
+                                              label=_("wxWidgets compatibility"))
         self.for_version_prop.set_str_value(self.for_version)
         
         # ALB 2004-01-18
         self.access_functions['use_new_namespace'] = (
             self.get_use_old_namespace, self.set_use_old_namespace)
         self.use_old_namespace_prop = CheckBoxProperty(
-            self, 'use_new_namespace', panel, 'Use old "from wxPython.wx"\n'
-            'import (python output only)')
+            self, 'use_new_namespace', lang_panels['python'],
+            _('Use old "from wxPython.wx"\nimport'))
         
         # `overwrite' property - added 2003-07-15
         self.overwrite = False
@@ -191,12 +189,12 @@ class Application(object):
         def set_overwrite(val): self.overwrite = bool(int(val))
         self.access_functions['overwrite'] = (get_overwrite, set_overwrite)
         self.overwrite_prop = CheckBoxProperty(self, 'overwrite', panel,
-                                               'Overwrite existing sources')
+                                               _('Overwrite existing sources'))
 
         self.outpath_prop = DialogProperty(self, "output_path", panel,
-                                           dialog)
+                                           dialog, label=_('Output path'))
         BTN_ID = wx.NewId()
-        btn = wx.Button(panel, BTN_ID, "Generate code")
+        btn = wx.Button(panel, BTN_ID, _("Generate code"))
 
         # layout of self.notebook
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -206,28 +204,42 @@ class Application(object):
         sizer.Add(self.use_gettext_prop.panel, 0, wx.EXPAND)
         szr = wx.BoxSizer(wx.HORIZONTAL)
         from widget_properties import _label_initial_width as _w
-        #label = wxGenStaticText(panel, -1, "Top window", size=(_w, -1))
-        label = wx.StaticText(panel, -1, "Top window", size=(_w, -1)) # ???
-        label.SetToolTip(wx.ToolTip("Top window"))
+        label = wx.StaticText(panel, -1, _("Top window"), size=(_w, -1))
+        label.SetToolTip(wx.ToolTip(_("Top window")))
         szr.Add(label, 2, wx.ALL|wx.ALIGN_CENTER, 3)
         szr.Add(self.top_win_prop, 5, wx.ALL|wx.ALIGN_CENTER, 3)
         sizer.Add(szr, 0, wx.EXPAND)
         sizer.Add(self.codegen_prop.panel, 0, wx.ALL|wx.EXPAND, 4)
         sizer.Add(self.codewriters_prop.panel, 0, wx.ALL|wx.EXPAND, 4)
         sizer.Add(self.for_version_prop.panel, 0, wx.ALL|wx.EXPAND, 4)
-        sizer.Add(self.use_old_namespace_prop.panel, 0, wx.EXPAND)
         sizer.Add(self.overwrite_prop.panel, 0, wx.EXPAND)
         sizer.Add(self.outpath_prop.panel, 0, wx.EXPAND)
+                
         sizer.Add(btn, 0, wx.ALL|wx.EXPAND, 5)
-        
-        panel.SetAutoLayout(True)
-        panel.SetSizer(sizer)
-        sizer.Layout()
-        sizer.Fit(panel)
-        h = panel.GetSize()[1]
-        self.notebook.AddPage(panel, "Application")
-        import math
-        panel.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
+
+        def do_add(l, p, s):
+            p.SetAutoLayout(True)
+            p.SetSizer(s)
+            s.Layout()
+            s.Fit(p)
+            h = p.GetSize()[1]
+            self.notebook.AddPage(p, l)
+            p.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
+            
+        do_add(_('Application'), panel, sizer)
+
+        # lang_panels['python']
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        sizer.Add(self.use_old_namespace_prop.panel, 0, wx.EXPAND)
+        do_add('Python', lang_panels['python'], sizer)
+
+        # lang_panels['C++']
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        sizer.Add(self.indent_mode_prop.panel, 0, wx.ALL|wx.EXPAND, 4)
+        sizer.Add(self.indent_amount_prop.panel, 0, wx.EXPAND)
+        sizer.Add(self.source_ext_prop.panel, 0, wx.EXPAND)
+        sizer.Add(self.header_ext_prop.panel, 0, wx.EXPAND)
+        do_add('C++', lang_panels['C++'], sizer)
 
         wx.EVT_BUTTON(btn, BTN_ID, self.generate_code)
         wx.EVT_CHOICE(self.top_win_prop, TOP_WIN_ID, self.set_top_window)
@@ -259,7 +271,7 @@ class Application(object):
         import locale
         locale.setlocale(locale.LC_ALL)
         try: return locale.nl_langinfo(locale.CODESET)
-        except AttributeError: return 'ISO-8859-1' # this is what I use...
+        except AttributeError: return 'ISO-8859-15' # this is what I use...
 
     def get_encoding(self):
         return self.encoding
@@ -267,20 +279,24 @@ class Application(object):
     def set_encoding(self, value):
         try: unicode('a', value)
         except LookupError, e:
-            wx.MessageBox(str(e), "Error", wx.OK|wx.CENTRE|wx.ICON_ERROR)
+            wx.MessageBox(str(e), _("Error"), wx.OK|wx.CENTRE|wx.ICON_ERROR)
             self.encoding_prop.set_value(self.encoding)
         else:
             self.encoding = value
 
     def set_language(self, value):
-        language = self.language = self.codewriters_prop.get_str_value()
+        language = self.codewriters_prop.get_str_value()
         ext = getattr(common.code_writers[language], 'default_extensions', [])
         wildcard = []
         for e in ext:
-            wildcard.append('%s files (*.%s)|*.%s' % (language.capitalize(),
+            wildcard.append(_('%s files (*.%s)|*.%s') % (language.capitalize(),
                                                       e, e))
-        wildcard.append('All files|*')
+        wildcard.append(_('All files|*'))
         self.outpath_prop.dialog.set_wildcard('|'.join(wildcard))
+        # check that the new language supports all the widgets in the tree
+        if self.language != language:
+            self.language = language
+            self.check_codegen()
 
     def get_language(self):
         return self.language #codewriters_prop.get_str_value()
@@ -356,6 +372,10 @@ class Application(object):
         self.name = "app"; self.name_prop.set_value("app")
         self.name_prop.toggle_active(False)
         self.codegen_opt = 0; self.codegen_prop.set_value(0)
+        self.indent_mode = 1
+        self.indent_amount = 4
+        self.cpp_source_ext = 'cpp'
+        self.cpp_header_ext = 'h'
         self.output_path = ""; self.outpath_prop.set_value("")
         # do not reset language, but call set_language anyway to update the
         # wildcard of the file dialog
@@ -381,7 +401,7 @@ class Application(object):
         
         self.notebook.Show(True)
         self.property_window.Layout()
-        self.property_window.SetTitle('Properties - <%s>' % self.name)
+        self.property_window.SetTitle(_('Properties - <%s>') % self.name)
         try: common.app_tree.select_item(self.node)
         except AttributeError: pass
 
@@ -391,19 +411,34 @@ class Application(object):
     def generate_code(self, *args, **kwds):
         preview = kwds.get('preview', False)
         if not self.output_path:
-            return wx.MessageBox("You must specify an output file\n"
-                                "before generating any code", "Error",
+            return wx.MessageBox(_("You must specify an output file\n"
+                                "before generating any code"), _("Error"),
                                 wx.OK|wx.CENTRE|wx.ICON_EXCLAMATION,
                                 self.notebook)
         if not preview and \
                ((self.name_prop.is_active() or self.klass_prop.is_active()) \
                 and self.top_win_prop.GetSelection() < 0):
-            return wx.MessageBox("Please select a top window "
-                                "for the application", "Error", wx.OK |
+            return wx.MessageBox(_("Please select a top window "
+                                "for the application"), _("Error"), wx.OK |
                                 wx.CENTRE | wx.ICON_EXCLAMATION, self.notebook)
-                
+
         from cStringIO import StringIO
-        out = StringIO()
+        class EncStringIO(object):
+            def __init__(self, encoding=None):
+                self.out = StringIO()
+                self.encoding = encoding
+
+            def write(self, data):
+                if self.encoding is not None and type(data) == type(u''):
+                    data = data.encode(self.encoding)
+                self.out.write(data)
+
+            def getvalue(self):
+                return self.out.getvalue()
+
+        # end of class EncStringIO
+        
+        out = EncStringIO(self.encoding)
         #common.app_tree.write(out) # write the xml onto a temporary buffer
         from xml_parse import CodeWriter
         try:
@@ -414,7 +449,8 @@ class Application(object):
                 common.code_writers[cw].use_new_namespace = True #False
                 overwrite = self.overwrite
                 self.overwrite = True
-            common.app_tree.write(out) # write the xml onto a temporary buffer
+            class_names = common.app_tree.write(out) # write the xml onto a
+                                                     # temporary buffer
             if not os.path.isabs(self.output_path) and \
                self.filename is not None:
                 out_path = os.path.join(os.path.dirname(self.filename),
@@ -422,27 +458,28 @@ class Application(object):
             else:
                 out_path = None
             CodeWriter(common.code_writers[cw], out.getvalue(), True,
-                       preview=preview, out_path=out_path)
+                       preview=preview, out_path=out_path,
+                       class_names=class_names)
             if preview and cw == 'python':
                 common.code_writers[cw].use_new_namespace = old
                 self.overwrite = overwrite
         except (IOError, OSError), msg:
-            wx.MessageBox("Error generating code:\n%s" % msg, "Error",
+            wx.MessageBox(_("Error generating code:\n%s") % msg, _("Error"),
                          wx.OK|wx.CENTRE|wx.ICON_ERROR)
         except Exception, msg:
-            import traceback; traceback.print_exc()
-            wx.MessageBox("An exception occurred while generating the code "
+            traceback.print_exc()
+            wx.MessageBox(_("An exception occurred while generating the code "
                          "for the application.\n"
                          "This is the error message associated with it:\n"
                          "        %s\n"
                          "For more details, look at the full traceback "
                          "on the console.\nIf you think this is a wxGlade bug,"
-                         " please report it." % msg, "Error",
+                         " please report it.") % msg, _("Error"),
                          wx.OK|wx.CENTRE|wx.ICON_ERROR)
         else:
             if not preview:
-                wx.MessageBox("Code generation completed successfully",
-                             "Information", wx.OK|wx.CENTRE|wx.ICON_INFORMATION)
+                wx.MessageBox(_("Code generation completed successfully"),
+                             _("Information"), wx.OK|wx.CENTRE|wx.ICON_INFORMATION)
 
     def get_name(self):
         if self.name_prop.is_active(): return self.name
@@ -477,7 +514,7 @@ class Application(object):
         widget.klass = '_%d_%s' % \
                        (random.randrange(10**8, 10**9), widget.klass)
             
-        real_path = self.output_path
+        self.real_output_path = self.output_path
         self.output_path = out_name[0]
         real_codegen_opt = self.codegen_opt
         real_language = self.language
@@ -520,26 +557,27 @@ class Application(object):
             def on_close(event):
                 frame.Destroy()
                 widget.preview_widget = None
-                widget.preview_button.SetLabel('Preview')
+                widget.preview_button.SetLabel(_('Preview'))
             wx.EVT_CLOSE(frame, on_close)
-            frame.SetTitle('<Preview> - %s' % frame.GetTitle())
+            frame.SetTitle(_('<Preview> - %s') % frame.GetTitle())
             # raise the frame
-            frame.Center()
+            frame.CenterOnScreen()
             frame.Show()
             # remove the temporary file (and the .pyc/.pyo ones too)
-            for ext in '', 'c', 'o':
+            for ext in '', 'c', 'o', '~':
                 name = self.output_path + ext
                 if os.path.isfile(name):
                     os.unlink(name)
         except Exception, e:
             #traceback.print_exc()
             widget.preview_widget = None
-            widget.preview_button.SetLabel('Preview')
-            wx.MessageBox("Problem previewing gui: %s" % str(e), "Error",
-                         wx.OK|wx.CENTRE|wx.ICON_EXCLAMATION, self.notebook)
+            widget.preview_button.SetLabel(_('Preview'))
+            wx.MessageBox(_("Problem previewing gui: %s") % str(e), _("Error"),
+                         wx.OK|wx.CENTRE|wx.ICON_EXCLAMATION)#, self.notebook)
         # restore app state
         widget.klass = widget_class_name
-        self.output_path = real_path
+        self.output_path = self.real_output_path
+        del self.real_output_path
         self.codegen_opt = real_codegen_opt
         self.language = real_language
         self.use_gettext = real_use_gettext
@@ -557,4 +595,37 @@ class Application(object):
         except:
             pass
 
+    def check_codegen(self, widget=None, language=None):
+        """\
+        Checks whether widget has a suitable code generator for the given
+        language (default: the current active language). If not, the user is
+        informed with a message.
+        """
+        if language is None: language = self.language
+        if widget is not None:
+            cname = common.class_names[widget.__class__.__name__]
+            if language != 'XRC':
+                ok = cname in common.code_writers[language].obj_builders
+            else:
+                # xrc is special...
+                xrcgen = common.code_writers['XRC']
+                ok = xrcgen.obj_builders.get(cname, None) is not \
+                     xrcgen.NotImplementedXrcObject
+            if not ok:
+                common.message(_('WARNING'),
+                               _('No %s code generator for %s (of type %s)'
+                               ' available'),
+                               language.capitalize(), widget.name, cname)
+        else:
+            # in this case, we check all the widgets in the tree
+            def check_rec(node):
+                if node.widget is not None:
+                    self.check_codegen(node.widget)
+                if node.children:
+                    for c in node.children:
+                        check_rec(c)
+            if common.app_tree.root.children:
+                for c in common.app_tree.root.children:
+                    check_rec(c)
+
 # end of class Application
