@@ -6,52 +6,71 @@
 # THIS PROGRAM COMES WITH NO WARRANTY
 
 import sys, os, os.path
-import common
+import common, config
 import cStringIO, re
 from xml_parse import XmlParsingError
 
 # these two globals must be defined for every code generator module
 language = 'C++'
-writer = sys.modules[__name__] # the writer is the module itself
+writer = sys.modules[__name__]
+"""\
+The writer is the module itself
+"""
 
-# default extensions for generated files: a list of file extensions
-default_extensions = ['h', 'cpp']
+default_extensions = ['h', 'hh', 'hpp', 'H', 'hxx',
+                      'cpp', 'cc', 'C', 'cxx', 'c++']
+"""\
+Default extensions for generated files: a list of file extensions
+"""
 
+classes = None
 """\
 dictionary that maps the lines of code of a class to the name of such class:
 the lines are divided in 3 categories: lines in the constructor,
 'set_properties' and 'do_layout'
 """
-classes = None
 
+obj_builders = {}
 """\
 dictionary of ``writers'' for the various objects. These are objects that must
 implement the WidgetHandler interface (see below)
 """
-obj_builders = {}
 
-# random number used to be sure that the replaced tags in the sources are
-# the right ones (see SourceFileContent and add_class)
 nonce = None
+"""\
+Random number used to be sure that the replaced tags in the sources are
+the right ones (see SourceFileContent and add_class)
+"""
 
-# lines common to all the generated files (include of <wx/wx.h>, ...)
 header_lines = []
+"""\
+Lines common to all the generated files (include of <wx/wx.h>, ...)
+"""
 
-# if True, generate a file for each custom class
 multiple_files = False
+"""\
+If True, generate a file for each custom class
+"""
 
-# if not None, they are the header and source file to write into
 output_header, output_source = None, None
-# if not None, name (without extension) of the file to write into
+"""\
+If not None, they are the header and source file to write into
+"""
+
 output_name = None
+"""\
+If not None, name (without extension) of the file to write into
+"""
 
-# if not None, it is the directory inside which the output files are saved
 out_dir = None
+"""\
+If not None, it is the directory inside which the output files are saved
+"""
 
-
-# ALB 2004-12-05: wx version we are generating code for
 for_version = (2, 4)
-
+"""\
+wx version we are generating code for
+"""
 
 class ClassLines:
     """\
@@ -76,7 +95,10 @@ class ClassLines:
                           # been generated
 
         # ALB 2004-12-08
-        self.event_handlers = [] # lines to bind events 
+        self.event_handlers = [] # lines to bind events
+
+        self.extra_code_h = [] # extra code to output
+        self.extra_code_cpp = []
 
 # end of class ClassLines
 
@@ -110,8 +132,8 @@ class SourceFileContent:
         and replaces the wxGlade blocks with tags that in turn will be replaced
         by the new wxGlade blocks
         """
-        self._build_untouched(self.name + '.h', True)
-        self._build_untouched(self.name + '.cpp', False)
+        self._build_untouched(self.name + header_extension, True)
+        self._build_untouched(self.name + source_extension, False)
 
     def _build_untouched(self, filename, is_header):
         class_name = None
@@ -129,7 +151,8 @@ class SourceFileContent:
         block_end = re.compile(r'^\s*//\s*end\s+wxGlade\s*$')
         # regexp to match event handlers
         # ALB 2004-12-08
-        event_handler = re.compile(r'^\s*void\s+([A-Za-z_]+\w*)\s*'
+        event_handler = re.compile(r'^\s*(?:virtual\s+)?'
+                                   'void\s+([A-Za-z_]+\w*)\s*'
                                    '\([A-Za-z_:0-9]+\s*&\s*\w*\)\s*;\s*'
                                    '//\s*wxGlade:\s*<event_handler>\s*$')
         decl_event_table = re.compile(r'^\s*DECLARE_EVENT_TABLE\s*\(\s*\)'
@@ -155,7 +178,7 @@ class SourceFileContent:
                 if end_index > comment_index: inside_comment = False
             if not is_header: result = None
             else: result = class_decl.match(line)
-            if not inside_comment and result is not None:
+            if not inside_comment and not inside_block and result is not None:
                 if class_name is None:
                     # this is the first class declared in the file: insert the
                     # new ones before this
@@ -238,23 +261,29 @@ class SourceFileContent:
 
 # end of class SourceFileContent
 
-# if not None, it is an instance of SourceFileContent that keeps info about
-# the previous version of the source to generate
 previous_source = None 
+"""\
+If not None, it is an instance of SourceFileContent that keeps info about
+the previous version of the source to generate
+"""
 
 
 def tabs(number):
-    return '    ' * number
+    return indent_symbol * indent_amount * number
 
-
-# if True, overwrite any previous version of the source file instead of
-# updating only the wxGlade blocks
 _overwrite = False
+"""\
+If True, overwrite any previous version of the source file instead of
+updating only the wxGlade blocks
+"""
 
-# if True, enable gettext support
 _use_gettext = False
+"""\
+If True, enable gettext support
+"""
 
 _quote_str_pattern = re.compile(r'\\[natbv"]?')
+
 def _do_replace(match):
     if match.group(0) == '\\': return '\\\\'
     else: return match.group(0)
@@ -281,11 +310,33 @@ def initialize(app_attrs):
     multi_files = app_attrs['option']
     
     global classes, header_lines, multiple_files, previous_source, nonce, \
-           _use_gettext, _overwrite, _last_generated_id
+           _use_gettext, _overwrite, _last_generated_id, \
+           _current_extra_code_h, _current_extra_code_cpp
     import time, random
 
     _last_generated_id = 1000
 
+    # Extensions based on Project options when set
+    global source_extension, header_extension
+    source_extension = app_attrs.get('source_extension', '.cpp')
+    header_extension = app_attrs.get('header_extension', '.h')
+
+    # Inentation level based on the project options
+    global indent_symbol, indent_amount
+    try:
+        indent_symbol = app_attrs['indent_symbol']
+        if indent_symbol == 'tab':
+            indent_symbol = '\t'
+        elif indent_symbol == 'space':
+            indent_symbol = ' '
+        else:
+            indent_symbol = ' '
+    except (KeyError, ValueError):
+        indent_symbol = ' '
+
+    try: indent_amount = int(app_attrs['indent_amount'])
+    except (KeyError, ValueError): indent_amount = 4
+
     try: _use_gettext = int(app_attrs['use_gettext'])
     except (KeyError, ValueError): _use_gettext = False
 
@@ -312,12 +363,21 @@ def initialize(app_attrs):
     header_lines = ['// -*- C++ -*- generated by wxGlade %s on %s%s\n\n' % \
                     (common.version, time.asctime(), common.generated_from()),
                     '#include <wx/wx.h>\n', '#include <wx/image.h>\n']
+    if not config.preferences.write_timestamp:
+        header_lines[0] = '// -*- C++ -*- generated by wxGlade %s%s\n\n' % \
+                          (common.version, common.generated_from())
+
+    # extra lines to generate (see the 'extracode' property of top-level
+    # widgets)
+    _current_extra_code_h = []
+    _current_extra_code_cpp = []
+    
     multiple_files = multi_files
     if not multiple_files:
         global output_header, output_source, output_name
         name, ext = os.path.splitext(out_path)
         output_name = name
-        if not _overwrite and os.path.isfile(name + '.h'):
+        if not _overwrite and os.path.isfile(name + header_extension):
             # the file exists, we must keep all the lines not inside a wxGlade
             # block. NOTE: this may cause troubles if out_path is not a valid
             # C++ file, so be careful!
@@ -330,7 +390,8 @@ def initialize(app_attrs):
                 output_header.write(line)
                 #output_source.write(line)
             # isolation directives
-            oh = os.path.basename(name + '.h').upper().replace('.', '_')
+            oh = os.path.basename(name + header_extension).upper().replace(
+                '.', '_')
             # extra headers
             #for val in _obj_headers.itervalues():
 ##             for handler in obj_builders.itervalues():
@@ -341,10 +402,13 @@ def initialize(app_attrs):
 
             output_header.write('\n#ifndef %s\n#define %s\n' % (oh, oh))
             output_header.write('\n')
+            # write the tag to store extra code
+            output_header.write('\n<%swxGlade replace  extracode>\n' % nonce)
 
             output_source.write(header_lines[0])
             output_source.write('#include "%s%s"\n\n' % \
-                                (os.path.basename(name), '.h'))
+                                (os.path.basename(name), header_extension))
+            output_source.write('<%swxGlade replace  extracode>\n\n' % nonce)
     else:
         previous_source = None
         global out_dir
@@ -368,6 +432,19 @@ def finalize():
         header_content = previous_source.header_content.replace(tag, code)
         extra_source = "".join([ c[1] for c in previous_source.new_classes])
         source_content = previous_source.source_content
+
+        # extra code (see the 'extracode' property of top-level widgets)
+        tag = '<%swxGlade replace  extracode>' % nonce
+        code = "\n".join(['// begin wxGlade: ::extracode'] +
+                         _current_extra_code_h +
+                         ['// end wxGlade\n'])
+        header_content = header_content.replace(tag, code)
+        code = "\n".join(['// begin wxGlade: ::extracode'] +
+                         _current_extra_code_cpp +
+                         ['// end wxGlade\n'])
+        source_content = source_content.replace(tag, code)
+        # --------------------------------------------------------------
+        
         # now remove all the remaining <123415wxGlade ...> tags from the
         # source: this may happen if we're not generating multiple files,
         # and one of the container class names is changed
@@ -411,19 +488,21 @@ def finalize():
                           source_content)
         for tag in tags:
             source_content = source_content.replace(tag, "")
-        
+
         # write the new file contents to disk
-        common.save_file(previous_source.name + '.h', header_content,
-                         'codegen')
-        common.save_file(previous_source.name + '.cpp',
-                         source_content + '\n\n' + extra_source,
-                         'codegen')
+        common.save_file(previous_source.name + header_extension,
+                         header_content, 'codegen')
+        if extra_source:
+            extra_source = '\n\n' + extra_source
+        common.save_file(previous_source.name + source_extension,
+                         source_content + extra_source, 'codegen')
         
     elif not multiple_files:
         oh = os.path.basename(output_name).upper() + '_H'
         output_header.write('\n#endif // %s\n' % oh)
         # write the list of include files
         header_content = output_header.getvalue()
+        source_content = output_source.getvalue()
         tags = re.findall('<%swxGlade replace  dependencies>' %
                           nonce, header_content)
         deps = []
@@ -438,9 +517,22 @@ def finalize():
         tmp.append("// end wxGlade\n")
         header_content = header_content.replace(
             '<%swxGlade replace  dependencies>' % nonce, "".join(tmp))
+
+        # extra code (see the 'extracode' property of top-level widgets)
+        tag = '<%swxGlade replace  extracode>' % nonce
+        code = "\n".join(['// begin wxGlade: ::extracode'] +
+                         _current_extra_code_h +
+                         ['// end wxGlade\n'])
+        header_content = header_content.replace(tag, code)
+        code = "\n".join(['// begin wxGlade: ::extracode'] +
+                         _current_extra_code_cpp +
+                         ['// end wxGlade\n'])
+        source_content = source_content.replace(tag, code)
+        # --------------------------------------------------------------        
             
-        common.save_file(output_name + '.h', header_content, 'codegen')
-        common.save_file(output_name + '.cpp', output_source.getvalue(),
+        common.save_file(output_name + header_extension, header_content,
+                         'codegen')
+        common.save_file(output_name + source_extension, source_content,
                          'codegen')
 
 
@@ -489,6 +581,31 @@ def add_object(top_obj, sub_obj):
                 for event, handler in sub_obj.properties['events'].iteritems():
                     klass.event_handlers.append((id, event, handler))
             # ---------------------------------------------------------------
+            # try to see if there's some extra code to add to this class
+            extra_code = getattr(builder, 'extracode',
+                                 sub_obj.properties.get('extracode', ""))
+            if extra_code:
+                extra_code = re.sub(r'\\n', '\n', extra_code)
+                extra_code = re.split(re.compile(r'^###\s*$', re.M),
+                                      extra_code, 1)
+                klass.extra_code_h.append(extra_code[0])
+                if len(extra_code) > 1:
+                    klass.extra_code_cpp.append(extra_code[1])
+                # if we are not overwriting existing source, warn the user
+                # about the presence of extra code
+                if multiple_files:
+                    warn = False
+                else:
+                    warn = previous_source is not None
+                if warn:
+                    common.message(
+                        'WARNING',
+                        '%s has extra code, but you are '
+                        'not overwriting existing sources: please check '
+                        'that the resulting code is correct!' % \
+                        sub_obj.name)
+            # -----------------------------------------------------------
+            
 
             klass.ids.extend(ids)
             if sub_obj.klass != 'spacer':
@@ -548,7 +665,8 @@ def add_class(code_obj):
         # let's see if the file to generate exists, and in this case
         # create a SourceFileContent instance
         filename = os.path.join(out_dir,
-                                code_obj.klass.replace('::', '_') + '.h')
+                                code_obj.klass.replace('::', '_') +
+                                header_extension)
         if _overwrite or not os.path.exists(filename):
             prev_src = None
         else:
@@ -587,6 +705,28 @@ def add_class(code_obj):
         print code_obj
         raise # this shouldn't happen
 
+    # try to see if there's some extra code to add to this class
+    extra_code = getattr(builder, 'extracode',
+                         code_obj.properties.get('extracode', ""))
+    if extra_code:
+        extra_code = re.sub(r'\\n', '\n', extra_code)
+        extra_code = re.split(re.compile(r'^###\s*$', re.M), extra_code, 1)
+        classes[code_obj.klass].extra_code_h.append(extra_code[0])
+        if len(extra_code) > 1:
+            classes[code_obj.klass].extra_code_cpp.append(extra_code[1])
+        if not is_new:
+            common.message('WARNING', '%s has extra code, but you are '
+                           'not overwriting existing sources: please check '
+                           'that the resulting code is correct!' % \
+                           code_obj.name)
+
+    if not multiple_files and extra_code:
+        _current_extra_code_h.append("".join(
+            classes[code_obj.klass].extra_code_h[::-1]))
+        _current_extra_code_cpp.append("".join(
+            classes[code_obj.klass].extra_code_cpp[::-1]))
+    #------------------------------------------------------------
+
     default_sign = [('wxWindow*', 'parent'), ('int', 'id')]
 ##     sign = obj_constructors.get(code_obj.base, default_sign)
     sign = getattr(builder, 'constructor', default_sign)
@@ -609,9 +749,25 @@ def add_class(code_obj):
     if hasattr(builder, 'get_events'):
         event_handlers.extend(builder.get_events(code_obj))
 
+    # ALB 2007-08-31 custom base classes support
+    custom_base = getattr(code_obj, 'custom_base',
+                          code_obj.properties.get('custom_base', None))
+    if custom_base and not custom_base.strip():
+        custom_base = None
+
+    if not is_new and custom_base is not None:
+        # custom base classes set, but "overwrite existing sources" not
+        # set. Issue a warning about this
+        common.message('WARNING', '%s has custom base classes, but you are '
+                       'not overwriting existing sources: please check that '
+                       'the resulting code is correct!' % code_obj.name)        
+
     if is_new:
         # header file
-        hwrite('\nclass %s: public %s {\n' % (code_obj.klass, code_obj.base))
+        base = code_obj.base
+        if custom_base is not None:
+            base = ", public ".join([b.strip() for b in custom_base.split(',')])
+        hwrite('\nclass %s: public %s {\n' % (code_obj.klass, base))
         hwrite('public:\n')
         # the first thing to add it the enum of the various ids
         hwrite(tabs(1) + '// begin wxGlade: %s::ids\n' % code_obj.klass)
@@ -656,7 +812,10 @@ def add_class(code_obj):
                     win_id, event, handler = tpl
                     evt_type = 'wxCommandEvent'
                 if handler not in already_there:
-                    hwrite(t + 'void %s(%s &event); '
+                    # Sebastien JEFFROY & Steve MULLER contribution
+                    # Adding virtual attribute permits to derivate from the
+                    # class generated by wxGlade
+                    hwrite(t + 'virtual void %s(%s &event); '
                            '// wxGlade: <event_handler>\n' %
                            (handler, evt_type))
                     already_there[handler] = 1
@@ -680,8 +839,9 @@ def add_class(code_obj):
         tag = '<%swxGlade replace %s ids>' % (nonce, code_obj.klass)
         if prev_src.header_content.find(tag) < 0:
             # no ids tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade ids block not found," \
-                  " ids declarations code NOT generated"
+            common.message("WARNING", "wxGlade ids block not found for %s," \
+                           " ids declarations code NOT generated" % \
+                           code_obj.name)
         else:
             prev_src.header_content = prev_src.header_content.\
                                       replace(tag, "".join(header_buffer))
@@ -693,8 +853,10 @@ def add_class(code_obj):
         tag = '<%swxGlade replace %s methods>' % (nonce, code_obj.klass)
         if prev_src.header_content.find(tag) < 0:
             # no methods tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade methods block not found," \
-                  " methods declarations code NOT generated"
+            common.message("WARNING",
+                           "wxGlade methods block not found for %s," \
+                           " methods declarations code NOT generated" % \
+                           code_obj.name)
         else:
             prev_src.header_content = prev_src.header_content.\
                                       replace(tag, "".join(header_buffer))
@@ -707,8 +869,10 @@ def add_class(code_obj):
         tag = '<%swxGlade replace %s attributes>' % (nonce, code_obj.klass)
         if prev_src.header_content.find(tag) < 0:
             # no attributes tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade attributes block " \
-                  "not found, attributes declarations code NOT generated"
+            common.message("WARNING",
+                           "wxGlade attributes block " \
+                           "not found for %s, attributes declarations code " \
+                           "NOT generated" % code_obj.name)
         else:
             prev_src.header_content = prev_src.header_content.\
                                       replace(tag, "".join(header_buffer))
@@ -726,7 +890,10 @@ def add_class(code_obj):
                     win_id, event, handler = tpl
                     evt_type = 'wxCommandEvent'
                 if handler not in already_there:
-                    hwrite(t + 'void %s(%s &event); // wxGlade: '
+                    # Sebastien JEFFROY & Steve MULLER contribution :
+                    # Adding virtual attribute permits to derivate from the
+                    # class generated by wxGlade
+                    hwrite(t + 'virtual void %s(%s &event); // wxGlade: '
                            '<event_handler>\n' % (handler, evt_type))
                     already_there[handler] = 1
             if code_obj.klass not in prev_src.event_table_def:
@@ -735,8 +902,9 @@ def add_class(code_obj):
         tag = '<%swxGlade event_handlers %s>' % (nonce, code_obj.klass)
         if prev_src.header_content.find(tag) < 0:
             # no attributes tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade events block " \
-                  "not found, event table code NOT generated"
+            common.message("WARNING", "wxGlade events block " \
+                           "not found for %s, event table code NOT generated" %
+                           code_obj.name)
         else:
             prev_src.header_content = prev_src.header_content.\
                                       replace(tag, "".join(header_buffer))
@@ -750,10 +918,18 @@ def add_class(code_obj):
     
     # constructor
     if is_new:
-        swrite('\n%s::%s(%s):\n%s%s(%s)\n{\n' % (code_obj.klass,
-                                                 code_obj.klass,
-                                                 sign_decl2, tabs(1),
-                                                 code_obj.base, sign_inst))
+        base = "%s(%s)" % (code_obj.base, sign_inst)
+        if custom_base:
+            bases = [b.strip() for b  in custom_base.split(',')]
+            if bases:
+                base = "%s(%s)" % (bases[0], sign_inst)
+                rest = ", ".join([b + "()" for b in bases[1:]])
+                if rest: base += ", " + rest
+            
+        swrite('\n%s::%s(%s):\n%s%s\n{\n' % (code_obj.klass,
+                                             code_obj.klass,
+                                             sign_decl2, tabs(1), base))
+##                                                  code_obj.base, sign_inst))
     swrite(tabs(1) + '// begin wxGlade: %s::%s\n' % (code_obj.klass,
                                                      code_obj.klass))
     tab = tabs(1)
@@ -783,9 +959,9 @@ def add_class(code_obj):
                                              code_obj.klass)
         if prev_src.source_content.find(tag) < 0:
             # no constructor tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade %s::%s block not found," \
-                  " relative code NOT generated" % (code_obj.klass,
-                                                    code_obj.klass)
+            common.message("WARNING", "wxGlade %s::%s block not found," \
+                           " relative code NOT generated" % (code_obj.klass,
+                                                             code_obj.klass))
         else:
             prev_src.source_content = prev_src.source_content.\
                                       replace(tag, "".join(source_buffer))
@@ -818,9 +994,9 @@ def add_class(code_obj):
             tag = '<%swxGlade replace %s event_table>' % (nonce, code_obj.klass)
             if prev_src.source_content.find(tag) < 0:
                 # no constructor tag found, issue a warning and do nothing
-                print >> sys.stderr, "WARNING: wxGlade %s::event_table block " \
-                      "not found, relative code NOT generated" % \
-                      (code_obj.klass)
+                common.message("WARNING", "wxGlade %s::event_table block " \
+                               "not found, relative code NOT generated" % \
+                               (code_obj.klass))
             else:
                 prev_src.source_content = prev_src.source_content.\
                                           replace(tag, "".join(source_buffer))
@@ -842,8 +1018,8 @@ def add_class(code_obj):
                 swrite('\n\nvoid %s::%s(%s &event)\n{\n' % \
                        (code_obj.klass, handler, evt_type))
                 swrite(tab + 'event.Skip();\n')
-                swrite(tab + 'std::cout<<"Event handler (%s::%s) not '
-                       'implemented yet"<<std::endl; //notify the user '
+                swrite(tab + 'wxLogDebug(wxT("Event handler (%s::%s) not '
+                       'implemented yet")); //notify the user '
                        'that he hasn\'t implemented the event handler yet\n' % \
                        (code_obj.klass, handler))
                 swrite('}\n')
@@ -859,9 +1035,9 @@ def add_class(code_obj):
                   (nonce, code_obj.klass)
             if prev_src.source_content.find(tag) < 0:
                 # no constructor tag found, issue a warning and do nothing
-                print >> sys.stderr, "WARNING: wxGlade %s event handlers " \
-                      "marker not found, relative code NOT generated" % \
-                      (code_obj.klass)
+                common.message("WARNING", "wxGlade %s event handlers " \
+                               "marker not found, relative code NOT generated" \
+                               % (code_obj.klass))
             else:
                 prev_src.source_content = prev_src.source_content.\
                                           replace(tag, "".join(source_buffer))
@@ -898,8 +1074,9 @@ def add_class(code_obj):
         tag = '<%swxGlade replace %s set_properties>' % (nonce, code_obj.klass)
         if prev_src.source_content.find(tag) < 0:
             # no set_properties tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade %s::set_properties block "\
-                  "not found, relative code NOT generated" % (code_obj.klass)
+            common.message("WARNING", "wxGlade %s::set_properties block "\
+                           "not found, relative code NOT generated" % \
+                           (code_obj.klass))
         else:
             prev_src.source_content = prev_src.source_content.\
                                       replace(tag, "".join(source_buffer))
@@ -934,8 +1111,9 @@ def add_class(code_obj):
         tag = '<%swxGlade replace %s do_layout>' % (nonce, code_obj.klass)
         if prev_src.source_content.find(tag) < 0:
             # no do_layout tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade %s::do_layout block "\
-                  "not found, relative code NOT generated" % (code_obj.klass)
+            common.message("WARNING", "wxGlade %s::do_layout block "\
+                           "not found, relative code NOT generated" %
+                           (code_obj.klass))
         else:
             prev_src.source_content = prev_src.source_content.\
                                       replace(tag, "".join(source_buffer))
@@ -977,16 +1155,41 @@ def add_class(code_obj):
             tag = '<%swxGlade replace  dependencies>' % nonce
             prev_src.header_content = prev_src.header_content.\
                                       replace(tag, "".join(deps))
+
+            # insert the extra code of this class
+            extra_code_h = "".join(classes[code_obj.klass].extra_code_h[::-1])
+            extra_code_cpp = \
+                           "".join(classes[code_obj.klass].extra_code_cpp[::-1])
+            # if there's extra code but we are not overwriting existing
+            # sources, warn the user
+            if extra_code_h or extra_code_cpp:
+                common.message('WARNING', '%s (or one of its chilren) has '
+                               'extra code classes, but you are '
+                               'not overwriting existing sources: please check '
+                               'that the resulting code is correct!' % \
+                               code_obj.name)
+            
+            extra_code_h = '// begin wxGlade: ::extracode\n%s\n' \
+                           '// end wxGlade\n' % extra_code_h
+            extra_code_cpp = '// begin wxGlade: ::extracode\n%s\n' \
+                             '// end wxGlade\n' % extra_code_cpp
+            tag = '<%swxGlade replace  extracode>' % nonce
+            prev_src.header_content = prev_src.header_content.replace(
+                tag, extra_code_h)
+            prev_src.source_content = prev_src.source_content.replace(
+                tag, extra_code_cpp)
             
             # store the new file contents to disk
             name = os.path.join(out_dir, code_obj.klass)
-            common.save_file(name + '.h', prev_src.header_content, 'codegen')
-            common.save_file(name + '.cpp', prev_src.source_content, 'codegen')
+            common.save_file(name + header_extension, prev_src.header_content,
+                             'codegen')
+            common.save_file(name + source_extension, prev_src.source_content,
+                             'codegen')
             return
 
         # create the new source file
-        header_file = os.path.join(out_dir, code_obj.klass + '.h')
-        source_file = os.path.join(out_dir, code_obj.klass + '.cpp')
+        header_file = os.path.join(out_dir, code_obj.klass + header_extension)
+        source_file = os.path.join(out_dir, code_obj.klass + source_extension)
         hout = cStringIO.StringIO()
         sout = cStringIO.StringIO()
         # header file
@@ -1007,6 +1210,14 @@ def add_class(code_obj):
                 hwrite('#include %s\n' % module)
         hwrite('// end wxGlade\n')
         hwrite('\n')
+
+        # insert the extra code of this class
+        extra_code_h = "".join(classes[code_obj.klass].extra_code_h[::-1])
+        extra_code_h = '// begin wxGlade: ::extracode\n%s\n// end wxGlade\n' % \
+                       extra_code_h
+        hwrite(extra_code_h)
+        hwrite('\n')
+        
         # write the class body
         for line in header_buffer: hwrite(line)
         hwrite('\n#endif // %s\n' % hn)
@@ -1017,6 +1228,14 @@ def add_class(code_obj):
         #for line in header_lines: swrite(line)
         swrite(header_lines[0])
         swrite('#include "%s"\n\n' % os.path.basename(header_file))
+
+        # insert the extra code of this class
+        extra_code_cpp = "".join(classes[code_obj.klass].extra_code_cpp[::-1])
+        extra_code_cpp = '// begin wxGlade: ::extracode\n%s\n' \
+                         '// end wxGlade\n' % extra_code_cpp
+        swrite(extra_code_cpp)
+        swrite('\n')
+        
         # write the class implementation
         for line in source_buffer: swrite(line)
 
@@ -1033,6 +1252,13 @@ def add_class(code_obj):
         for line in header_buffer: hwrite(line)
         swrite = output_source.write
         for line in source_buffer: swrite(line)
+        # extra code
+        ## if classes[code_obj.klass].extra_code_h:
+        ##     _current_extra_code_h.extend(
+        ##         classes[code_obj.klass].extra_code_h[::-1])
+        ## if classes[code_obj.klass].extra_code_cpp:
+        ##     _current_extra_code_cpp.extend(
+        ##         classes[code_obj.klass].extra_code_h[::-1])
 
 
 def add_app(app_attrs, top_win_class):
@@ -1222,6 +1448,15 @@ def generate_code_hidden(obj):
         return self + 'Hide();\n'
 
 
+def generate_code_extraproperties(obj):
+    self = _get_code_name(obj)
+    prop = obj.properties['extraproperties']
+    ret = []
+    for name in sorted(prop):
+        ret.append(self + 'Set%s(%s);\n' % (name, prop[name]))
+    return ret
+
+
 def generate_common_properties(widget):
     """\
     generates the code for various properties common to all widgets (background
@@ -1239,6 +1474,9 @@ def generate_common_properties(widget):
     if prop.get('disabled'): out.append(generate_code_disabled(widget))
     if prop.get('focused'): out.append(generate_code_focused(widget))
     if prop.get('hidden'): out.append(generate_code_hidden(widget))
+    # ALB 2007-09-01 extra properties
+    if prop.get('extraproperties') and not widget.preview:
+        out.extend(generate_code_extraproperties(widget))
     return out
 
 
@@ -1320,18 +1558,56 @@ class EventsPropertyHandler(object):
 # end of class EventsPropertyHandler
     
 
-# dictionary whose items are custom handlers for widget properties
+class ExtraPropertiesPropertyHandler(object):
+    def __init__(self):
+        self.props = {}
+        self.prop_name = None
+        self.curr_prop = []
+        
+    def start_elem(self, name, attrs):
+        if name == 'property':
+            name = attrs['name']
+            if name and name[0].islower():
+                name = name[0].upper() + name[1:]
+            self.prop_name = name
+
+    def end_elem(self, name, code_obj):
+        if name == 'property':
+            if self.prop_name and self.curr_prop:
+                self.props[self.prop_name] = ''.join(self.curr_prop)
+            self.prop_name = None
+            self.curr_prop = []
+        elif name == 'extraproperties':
+            code_obj.properties['extraproperties'] = self.props
+            return True # to remove this handler
+
+    def char_data(self, data):
+        data = data.strip()
+        if data:
+            self.curr_prop.append(data)
+
+# end of class ExtraPropertiesPropertyHandler
+
+
 _global_property_writers = { 'font': FontPropertyHandler,
                              'events': EventsPropertyHandler,
+                             'extraproperties': ExtraPropertiesPropertyHandler,
                              }
+"""\
+Dictionary whose items are custom handlers for widget properties
+"""
 
-# dictionary of dictionaries of property handlers specific for a widget
-# the keys are the class names of the widgets
-# Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
 _property_writers = {}
+"""\
+Dictionary of dictionaries of property handlers specific for a widget
+the keys are the class names of the widgets
+Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
+"""
 
-# dictionary of additional headers for objects
 _obj_headers = {}
+"""\
+Dictionary of additional headers for objects
+"""
 
 def get_property_handler(property_name, widget_name):
     try: cls = _property_writers[widget_name][property_name]
@@ -1358,14 +1634,14 @@ class WidgetHandler:
     Interface the various code generators for the widgets must implement
     """
 
-    """``signature'' of the widget's constructor"""
     constructor = []
+    """``signature'' of the widget's constructor"""
     
-    """
-    if not None, list of extra header file, in the form
+    extra_headers = []
+    """\
+    If not None, list of extra header file, in the form
     <header.h> or "header.h"
     """
-    extra_headers = []
     
     def get_code(self, obj):
         """\
