@@ -1,5 +1,5 @@
 # pl_codegen.py: perl code generator
-# $Id: pl_codegen.py,v 1.40 2007/03/09 07:33:37 agriggio Exp $
+# $Id: pl_codegen.py,v 1.41 2007/08/07 12:26:20 agriggio Exp $
 #
 # Copyright (c) 2002-2004 D.H. aka crazyinsomniac on sourceforge.net
 # License: MIT (see license.txt)
@@ -21,54 +21,74 @@ methods of the parent object.
 """
 
 import sys, os, os.path
-import common
+import common, config
 import cStringIO
 from xml_parse import XmlParsingError
 
 
 # these two globals must be defined for every code generator module
 language = 'perl'
-writer = sys.modules[__name__] # the writer is the module itself
-
-# default extensions for generated files: a list of file extensions
-default_extensions = ['pl','pm']
-
+writer = sys.modules[__name__]
 """\
-dictionary that maps the lines of code of a class to the name of such class:
+The writer is the module itself
+"""
+
+default_extensions = ['pl','pm']
+"""\
+Default extensions for generated files: a list of file extensions
+"""
+
+classes = None
+"""\
+Dictionary that maps the lines of code of a class to the name of such class:
 the lines are divided in 3 categories: '__init__', '__set_properties' and
 '__do_layout'
 """
-classes = None
 
-"""dictionary of ``writers'' for the various objects"""
 obj_builders = {}
-
 """\
-dictionary of ``property writer'' functions, used to set the properties of a
+Dictionary of ``writers'' for the various objects
+"""
+
+obj_properties = {}
+"""\
+Dictionary of ``property writer'' functions, used to set the properties of a
 toplevel object
 """
-obj_properties = {}
 
-# random number used to be sure that the replaced tags in the sources are
-# the right ones (see SourceFileContent and add_class)
 nonce = None
+"""\
+Random number used to be sure that the replaced tags in the sources are
+the right ones (see SourceFileContent and add_class)
+"""
 
-# lines common to all the generated files (use Wx [:everything], ...)
 header_lines = []
+"""\
+Lines common to all the generated files (use Wx [:everything], ...)
+"""
 
-# if True, generate a file for each custom class
 multiple_files = False
+"""\
+If True, generate a file for each custom class
+"""
 
-# if not None, it is the single source file to write into
 output_file = None
-# if not None, it is the directory inside which the output files are saved
+"""\
+If not None, it is the single source file to write into
+"""
+
 out_dir = None
+"""\
+If not None, it is the directory inside which the output files are saved
+"""
 
 def cn(class_name):
     if class_name[:2] == 'wx': 
         return 'Wx::' + class_name[2:]
     elif class_name[:4] == 'EVT_':
         return 'Wx::' + class_name
+    else:
+        return class_name
 
 class ClassLines:
     """\
@@ -221,16 +241,22 @@ class SourceFileContent:
 
 # end of class SourceFileContent
 
-# if not None, it is an instance of SourceFileContent that keeps info about
-# the previous version of the source to generate
 previous_source = None 
+"""\
+If not None, it is an instance of SourceFileContent that keeps info about
+the previous version of the source to generate
+"""
 
-# if True, overwrite any previous version of the source file instead of
-# updating only the wxGlade blocks
 _overwrite = False
+"""\
+If True, overwrite any previous version of the source file instead of
+updating only the wxGlade blocks
+"""
 
-# if True, enable gettext support
 _use_gettext = False
+"""\
+If True, enable gettext support
+"""
 
 import re
 _quote_str_re = re.compile( r'\\(?![nrt])' )
@@ -250,6 +276,24 @@ def quote_str(s):
     else:
         return '"' + s + '"'
 
+def quote_key(s):
+    """\
+    returns a possibly quoted version of 's', suitable to insert in a perl
+    source file as a hask key. Takes care also of gettext support
+    """
+    if not s: return '""'
+    t = s
+    s = _quote_str_re.sub(r'\\\\', s )
+    s = s.replace('"', r'\"')
+    s = s.replace('$', r'\$')
+    s = s.replace('@', r'\@')
+    if _use_gettext:
+        return '_T("' + s + '")'
+    if t == s and s.find(' ') < 0:
+        return s;
+    else:
+        return '"' + s + '"'
+
 def quote_path(s):
     """\
     escapes all " and \ , thus making a path suitable to
@@ -264,11 +308,10 @@ def quote_path(s):
 def initialize(app_attrs): 
     """\
     Writer initialization function.
-    - app_attrs: dict of attributes of the application. The following two
-                 are always present:
-           path: output path for the generated code (a file if multi_files is
-                 False, a dir otherwise)
-         option: if True, generate a separate file for each custom class
+
+    @keyword path: Output path for the generated code (a file if multi_files is
+                   False, a dir otherwise)
+    @keyword option: If True, generate a separate file for each custom class
     """
 
     out_path = app_attrs['path']
@@ -278,34 +321,34 @@ def initialize(app_attrs):
            _current_extra_modules, _use_gettext, _overwrite
     import time, random
 
-# scan widgets.txt for widgets, load perl_codegen's
-    _widgets_dir = os.path.join(common.wxglade_path, 'widgets')
-    widgets_file = os.path.join(_widgets_dir, 'widgets.txt')
-    if not os.path.isfile(widgets_file):
-        print >> sys.stderr, "widgets file (%s) doesn't exist" % widgets_file
-        return
-    import sys
-    sys.path.append(_widgets_dir)
-    modules = open(widgets_file)
-    for line in modules:
-        module_name = line.strip()
-        if not module_name or module_name.startswith('#'): continue
-        module_name = module_name.split('#')[0].strip()
-        try:
-            m = __import__(
-                module_name + '.perl_codegen', {}, {}, ['initialize'])
-            m.initialize()
-        except (ImportError, AttributeError):
-            print 'ERROR loading "%s"' % module_name
-            import traceback;
-            traceback.print_exc()
-#        else:
-#            print 'initialized perl generator for ', module_name
-    modules.close()
+##     # scan widgets.txt for widgets, load perl_codegen's
+##     _widgets_dir = os.path.join(common.wxglade_path, 'widgets')
+##     widgets_file = os.path.join(_widgets_dir, 'widgets.txt')
+##     if not os.path.isfile(widgets_file):
+##         print >> sys.stderr, "widgets file (%s) doesn't exist" % widgets_file
+##         return
+##     import sys
+##     sys.path.append(_widgets_dir)
+##     modules = open(widgets_file)
+##     for line in modules:
+##         module_name = line.strip()
+##         if not module_name or module_name.startswith('#'): continue
+##         module_name = module_name.split('#')[0].strip()
+##         try:
+##             m = __import__(
+##                 module_name + '.perl_codegen', {}, {}, ['initialize'])
+##             m.initialize()
+##         except (ImportError, AttributeError):
+##             print 'ERROR loading "%s"' % module_name
+##             import traceback;
+##             traceback.print_exc()
+## ##        else:
+## ##            print 'initialized perl generator for ', module_name
+##     modules.close()
 
-   # ...then, the sizers
-    import edit_sizers.perl_sizers_codegen
-    edit_sizers.perl_sizers_codegen.initialize()  
+##     # ...then, the sizers
+##     import edit_sizers.perl_sizers_codegen
+##     edit_sizers.perl_sizers_codegen.initialize()  
 
     try:
         _use_gettext = int(app_attrs['use_gettext'])
@@ -334,10 +377,14 @@ def initialize(app_attrs):
     _current_extra_modules = {}
     header_lines = [
         '# generated by wxGlade %s on %s%s\n' % \
-        (common.version, time.asctime(), common.generated_from()),
+        (common.version, time.asctime(), common.generated_from()), 
         '# To get wxPerl visit http://wxPerl.sourceforge.net/\n\n',
         'use Wx 0.15 qw[:allclasses];\nuse strict;\n'
     ]
+    if not config.preferences.write_timestamp:
+        header_lines[0] = '# generated by wxGlade %s%s\n' % \
+                          (common.version, common.generated_from())
+    
 
     multiple_files = multi_files
     if not multiple_files:
@@ -1078,6 +1125,15 @@ def generate_code_hidden(obj):
         return  self + '->Show(0);\n' # Hide() is not available
     
 
+def generate_code_extraproperties(obj):
+    self = _get_code_name(obj)
+    prop = obj.properties['extraproperties']
+    ret = []
+    for name in sorted(prop):
+        ret.append(self + '->Set%s(%s);\n' % (name, prop[name]))
+    return ret
+
+
 def generate_common_properties(widget):
     """\
     generates the code for various properties common to all widgets (background
@@ -1096,6 +1152,9 @@ def generate_common_properties(widget):
     if prop.get('disabled'): out.append(generate_code_disabled(widget))
     if prop.get('focused'): out.append(generate_code_focused(widget))
     if prop.get('hidden'): out.append(generate_code_hidden(widget))
+    # ALB 2007-09-01 extra properties
+    if prop.get('extraproperties') and not widget.preview:
+        out.extend(generate_code_extraproperties(widget))
     return out
 
 
@@ -1181,22 +1240,62 @@ class EventsPropertyHandler(object):
 # end of class EventsPropertyHandler
     
 
-# dictionary whose items are custom handlers for widget properties
+class ExtraPropertiesPropertyHandler(object):
+    def __init__(self):
+        self.props = {}
+        self.prop_name = None
+        self.curr_prop = []
+        
+    def start_elem(self, name, attrs):
+        if name == 'property':
+            name = attrs['name']
+            if name and name[0].islower():
+                name = name[0].upper() + name[1:]
+            self.prop_name = name
+
+    def end_elem(self, name, code_obj):
+        if name == 'property':
+            if self.prop_name and self.curr_prop:
+                self.props[self.prop_name] = ''.join(self.curr_prop)
+            self.prop_name = None
+            self.curr_prop = []
+        elif name == 'extraproperties':
+            code_obj.properties['extraproperties'] = self.props
+            return True # to remove this handler
+
+    def char_data(self, data):
+        data = data.strip()
+        if data:
+            self.curr_prop.append(data)
+
+# end of class ExtraPropertiesPropertyHandler
+
+
 _global_property_writers = { 'font': FontPropertyHandler,
-                             'events': EventsPropertyHandler, }
+                             'events': EventsPropertyHandler,
+                             'extraproperties': ExtraPropertiesPropertyHandler,
+                             }
+"""\
+Dictionary whose items are custom handlers for widget properties
+"""
 
-# dictionary of dictionaries of property handlers specific for a widget
-# the keys are the class names of the widgets
-# Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
 _property_writers = {}
+"""\
+Dictionary of dictionaries of property handlers specific for a widget
+the keys are the class names of the widgets
+Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
+"""
 
-
-# map of widget class names to a list of extra modules needed for the
-# widget. Example: 'wxGrid': 'use Wx::Grid;\n'
 _widget_extra_modules = {}
+"""\
+Map of widget class names to a list of extra modules needed for the
+widget. Example: 'wxGrid': 'use Wx::Grid;\n'
+"""
 
-# set of lines of extra modules to add to the current file
 _current_extra_modules = {} 
+"""\
+Set of lines of extra modules to add to the current file
+"""
 
 def get_property_handler(property_name, widget_name):
     try: cls = _property_writers[widget_name][property_name]
@@ -1222,11 +1321,11 @@ class WidgetHandler:
     Interface the various code generators for the widgets must implement
     """
     
-    """list of modules to import (eg. ['use Wx::Grid;\n'])"""
     import_modules = []
+    """list of modules to import (eg. ['use Wx::Grid;\n'])"""
     
-    """constructor signature ($self->SUPER::new(@stuff), see new_defaults )"""
     new_signature = []
+    """constructor signature ($self->SUPER::new(@stuff), see new_defaults )"""
 
     def get_code(self, obj):
         """\
@@ -1267,3 +1366,39 @@ class WidgetHandler:
 def add_widget_handler(widget_name, handler):
     obj_builders[widget_name] = handler
 
+
+
+def setup():
+    """\
+    Code generator setup function. This is called once, when the code
+    generator is loaded in wxglade.
+    """
+    import sys
+    # scan widgets.txt for widgets, load perl_codegen's
+    _widgets_dir = os.path.join(common.wxglade_path, 'widgets')
+    widgets_file = os.path.join(_widgets_dir, 'widgets.txt')
+    if not os.path.isfile(widgets_file):
+        print >> sys.stderr, "widgets file (%s) doesn't exist" % widgets_file
+        return
+    sys.path.append(_widgets_dir)
+    modules = open(widgets_file)
+    for line in modules:
+        module_name = line.strip()
+        if not module_name or module_name.startswith('#'): continue
+        module_name = module_name.split('#')[0].strip()
+        try:
+            m = __import__(
+                module_name + '.perl_codegen', {}, {}, ['initialize'])
+            m.initialize()
+        except (ImportError, AttributeError):
+            pass
+##             print 'ERROR loading "%s"' % module_name
+##             import traceback;
+##             traceback.print_exc()
+##         else:
+##             print 'initialized perl generator for ', module_name
+    modules.close()
+
+    # ...then, the sizers
+    import edit_sizers.perl_sizers_codegen
+    edit_sizers.perl_sizers_codegen.initialize()  
