@@ -1,5 +1,5 @@
 # edit_sizers.py: hierarchy of Sizers supported by wxGlade
-# $Id: edit_sizers.py,v 1.75 2007/03/27 07:02:06 agriggio Exp $
+# $Id: edit_sizers.py,v 1.80 2007/08/07 12:21:55 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
@@ -9,7 +9,7 @@ import wx
 from widget_properties import *
 from tree import Tree, WidgetTree
 import common, config, misc
-import math, sys, re
+import math, re
 
 class SizerSlot:
     "a window to represent a slot in a sizer"
@@ -42,7 +42,7 @@ class SizerSlot:
                 if evt_flags == flags and evt_key == key:
                     misc.wxCallAfter(function)
                     break
-            event.Skip()
+            #event.Skip()
         wx.EVT_KEY_DOWN(self.widget, on_key_down)
 
     def show_widget(self, yes):
@@ -85,12 +85,20 @@ class SizerSlot:
             if not self.sizer.is_virtual():
                 # we cannot remove items from virtual sizers
                 misc.append_item(self.menu, REMOVE_ID, _('Remove\tDel'),
-                                 'remove.xpm')
-            misc.append_item(self.menu, PASTE_ID, _('Paste\tCtrl+V'), 'paste.xpm')
+                                 wx.ART_DELETE)
+            misc.append_item(self.menu, PASTE_ID, _('Paste\tCtrl+V'),
+                             wx.ART_PASTE)
             def bind(method):
                 return lambda e: misc.wxCallAfter(method)
             wx.EVT_MENU(self.widget, REMOVE_ID, bind(self.remove))
             wx.EVT_MENU(self.widget, PASTE_ID, bind(self.clipboard_paste))
+
+            PREVIEW_ID = wx.NewId()
+            self.menu.AppendSeparator()
+            misc.append_item(self.menu, PREVIEW_ID, _('Preview'))
+            wx.EVT_MENU(self.widget, PREVIEW_ID, bind(self.preview_parent))
+
+        self.setup_preview_menu()
         self.widget.PopupMenu(self.menu, event.GetPosition())
 
     def remove(self, *args):
@@ -139,6 +147,7 @@ class SizerSlot:
             misc._currently_under_mouse = None
         if delete_widget and self.widget: self.widget.Destroy()
         if misc.focused_widget is self: misc.focused_widget = None
+        common.app_tree.app.saved = False # update the status of the app
 
     def update_pos(self, value):
         """\
@@ -147,6 +156,20 @@ class SizerSlot:
         """
         self.pos = value
 
+    def setup_preview_menu(self):
+        p = misc.get_toplevel_widget(self.sizer)
+        if p is not None:
+            item = list(self.menu.GetMenuItems())[-1]
+            if p.preview_is_visible():
+                item.SetText(_('Close preview') + ' (%s)\tCtrl+P' % p.name)
+            else:
+                item.SetText(_('Preview') + ' (%s)\tCtrl+P' % p.name)        
+
+    def preview_parent(self):
+        p = misc.get_toplevel_widget(self.sizer)
+        if p is not None:
+            p.preview(None)
+
 # end of class SizerSlot
 
 if 0: #wxPlatform != '__WXMAC__':
@@ -165,6 +188,8 @@ class SizerHandleButton(Button):
         self.sizer = sizer
         self.menu = menu
         self._rmenu = None
+        try: self.SetUseFocusIndicator(False)
+        except AttributeError: pass
 ##         # provide popup menu for removal
 ##         REMOVE_ID = wxNewId() 
 ##         self._rmenu = misc.wxGladePopupMenu(sizer.name)
@@ -193,7 +218,7 @@ class SizerHandleButton(Button):
                 if evt_flags == flags and evt_key == key:
                     misc.wxCallAfter(function)
                     break
-            event.Skip()
+            #event.Skip()
         wx.EVT_KEY_DOWN(self, on_key_down)
 
         def on_set_focus(event):
@@ -213,7 +238,7 @@ class SizerHandleButton(Button):
                 return lambda e: misc.wxCallAfter(method)
             #self._rmenu.Append(REMOVE_ID, 'Remove\tDel')
             misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
-                             'remove.xpm')
+                             wx.ART_DELETE)
             wx.EVT_MENU(self, REMOVE_ID, bind(self._remove))
             for item in self.menu:
                 id = wx.NewId()
@@ -222,8 +247,13 @@ class SizerHandleButton(Button):
                 if len(item) > 2: bmp = item[2]
                 misc.append_item(self._rmenu, id, item[0], bmp)
                 wx.EVT_MENU(self, id, bind(item[1]))
+            self._rmenu.AppendSeparator()
+            PREVIEW_ID = wx.NewId()
+            misc.append_item(self._rmenu, PREVIEW_ID, _('Preview'))
+            wx.EVT_MENU(self, PREVIEW_ID, bind(self.preview_parent))
             self.sizer._rmenu = self._rmenu
             del self.menu
+        self.setup_preview_menu()
         self.PopupMenu(self._rmenu, event.GetPosition())
 
     def _remove(self, *args):
@@ -243,6 +273,19 @@ class SizerHandleButton(Button):
         Button.Destroy(self)
         if misc.focused_widget is self: misc.focused_widget = None
 
+    def setup_preview_menu(self):
+        p = misc.get_toplevel_widget(self.sizer)
+        if p is not None:
+            item = list(self._rmenu.GetMenuItems())[-1]
+            if p.preview_is_visible():
+                item.SetText(_('Close preview') + ' (%s)\tCtrl+P' % p.name)
+            else:
+                item.SetText(_('Preview') + ' (%s)\tCtrl+P' % p.name)        
+
+    def preview_parent(self):
+        p = misc.get_toplevel_widget(self.sizer)
+        p.preview(None)
+
 # end of class SizerHandleButton
 
 
@@ -285,6 +328,7 @@ class SizerClassDialog:
         choices = [ b for a, b in self.choices if a != name ]
         self.dialog = wx.SingleChoiceDialog(self.parent, _("Select sizer type"),
                                            _("Select sizer type"), choices)
+        self.dialog.CenterOnScreen()
         return self.dialog.ShowModal()
 
     def get_value(self):
@@ -293,7 +337,7 @@ class SizerClassDialog:
 # end of class SizerClassDialog
             
 
-def change_sizer(old, new, which_page=0):
+def change_sizer(old, new, which_page=0, _hidden=[None]):
     """\
     changes 'old' sizer to 'new'
     Params:
@@ -350,10 +394,23 @@ def change_sizer(old, new, which_page=0):
             except (AttributeError, KeyError):
                 pass
     szr.show_widget(True, dont_set=True)
+
+    if _hidden[0] is None:
+        _hidden[0] = wx.Frame(None, -1, _("HIDDEN FRAME FOR CHANGE SIZER"))
+    
     for c in szr.children[1:]:
         widget = c.item
         widget.sizer = szr
         if not isinstance(widget, SizerSlot):
+            # ALB 2007-09-04. This is necessary as a workaround to a
+            # wx.StaticBoxSizer issue: it seems that the wx.StaticBox needs to
+            # come before any other widget managed by the wx.StaticBoxSizer in
+            # the GetChildren() list. Explicitly reparenting the widgets seems
+            # to solve the problem
+            if hasattr(widget.widget, 'GetParent'):
+                p = widget.widget.GetParent()
+                widget.widget.Reparent(_hidden[0])
+                widget.widget.Reparent(p)
             szr.widget.Insert(widget.pos, widget.widget,
                               int(widget.get_option()), widget.get_int_flag(),
                               int(widget.get_border()))
@@ -367,7 +424,6 @@ def change_sizer(old, new, which_page=0):
         if szr.sizer.widget:
             elem = szr.sizer.widget.GetChildren()[szr.pos]
             elem.SetSizer(szr.widget)
-    import common
     common.app_tree.change_node(szr.node, szr)
     old.toplevel = False
     szr.show_properties()
@@ -388,7 +444,7 @@ class InsertDialog(wx.Dialog):
     def __init__(self, max_val):
         wx.Dialog.__init__(self, None, -1, _("Select a position"))
         self.pos = 0
-        pos_prop = SpinProperty(self, 'position', self, r=(0, max_val))
+        pos_prop = SpinProperty(self, 'position', self, r=(0, max_val), label=_("position"))
         szr = wx.BoxSizer(wx.VERTICAL)
         szr.Add(pos_prop.panel, 0, wx.ALL|wx.EXPAND, 5)
         szr2 = wx.BoxSizer(wx.HORIZONTAL)
@@ -398,6 +454,7 @@ class InsertDialog(wx.Dialog):
         self.SetAutoLayout(True)
         self.SetSizer(szr)
         szr.Fit(self)
+        self.CenterOnScreen()
 
     def __getitem__(self, name):
         def set_pos(v): self.pos = int(v)
@@ -495,8 +552,10 @@ class SizerBase(Sizer):
             self.menu = [(_('Add slot'), self.add_slot),
                          (_('Insert slot...'), self.insert_slot)]
         #if not self.toplevel:
-        self.menu.extend([(_('Copy\tCtrl+C'), self.clipboard_copy,'copy.xpm'),
-                          (_('Cut\tCtrl+X'), self.clipboard_cut, 'cut.xpm')
+        self.menu.extend([(_('Copy\tCtrl+C'), self.clipboard_copy,
+                           wx.ART_COPY),
+                          (_('Cut\tCtrl+X'), self.clipboard_cut,
+                           wx.ART_CUT), 
                           ])
 
         self._btn = None # SizerHandleButton
@@ -526,6 +585,8 @@ class SizerBase(Sizer):
         # ALB 2004-08-11
         if not config.preferences.show_sizer_handle:
             self.widget.Show(self._btn, False)
+        if misc.focused_widget is self:
+            self.update_view(True)
 
     def _property_setup(self):
         """\
@@ -541,15 +602,15 @@ class SizerBase(Sizer):
             'class' : (lambda : self.klass, self.change) #lambda v: None)
             }
         if not self.toplevel:
-            self.access_functions['option'] = (self.get_option,self.set_option)
+            self.access_functions['option'] = (self.get_option, self.set_option)
             self.access_functions['flag'] = (self.get_flag, self.set_flag)
-            self.access_functions['border'] = (self.get_border,self.set_border)
+            self.access_functions['border'] = (self.get_border, self.set_border)
             self.access_functions['pos'] = (self.get_pos, self.set_pos)
 
-        self.name_prop = TextProperty(self, 'name', None)
+        self.name_prop = TextProperty(self, 'name', None, label=_('name'))
         #self.klass_prop = TextProperty(self, 'class', None, readonly=True)
         dialog = SizerClassDialog(self, None)
-        self.klass_prop = DialogProperty(self, 'class', None, dialog)
+        self.klass_prop = DialogProperty(self, 'class', None, dialog, label=_('class'))
         if not self.toplevel:
             prop = self.sizer_properties = {}
 
@@ -558,17 +619,14 @@ class SizerBase(Sizer):
                  LayoutPosProperty
             prop['option'] = LayoutOptionProperty(self, self.sizer)
 
-            flag_labels = ['#section#Border', 'wxALL',
+            flag_labels = ['#section#' + _('Border'), 'wxALL',
                            'wxLEFT', 'wxRIGHT', 'wxTOP', 'wxBOTTOM',
-                           '#section#Alignment', 'wxEXPAND', 'wxALIGN_RIGHT',
+                           '#section#' + _('Alignment'), 'wxEXPAND', 'wxALIGN_RIGHT',
                            'wxALIGN_BOTTOM', 'wxALIGN_CENTER_HORIZONTAL',
                            'wxALIGN_CENTER_VERTICAL', 'wxSHAPED',
                            'wxADJUST_MINSIZE' ]
             prop['flag'] = CheckListProperty(self, 'flag', None, flag_labels)
-            prop['border'] = SpinProperty(self, 'border', None, 0, (0, 1000))
-##             pos_p = prop['pos'] = SpinProperty(self, 'pos', None, 0, (0, 1000))
-##             def write(*args, **kwds): pass
-##             pos_p.write = write
+            prop['border'] = SpinProperty(self, 'border', None, 0, (0, 1000), label=_('border'))
             prop['pos'] = LayoutPosProperty(self, self.sizer)
 
     def set_containing_sizer(self, sizer):
@@ -612,10 +670,10 @@ class SizerBase(Sizer):
         sizer_tmp.Add(self.klass_prop.panel, 0, wx.EXPAND)
         if not self.toplevel:
             prop = self.sizer_properties
-            prop['option'].display(panel)
-            prop['flag'].display(panel)
-            prop['border'].display(panel)
             prop['pos'].display(panel)
+            prop['option'].display(panel)
+            prop['border'].display(panel)
+            prop['flag'].display(panel)
             sizer_tmp.Add(prop['pos'].panel, 0, wx.EXPAND)
             sizer_tmp.Add(prop['option'].panel, 0, wx.EXPAND)
             sizer_tmp.Add(prop['border'].panel, 0, wx.EXPAND)
@@ -701,6 +759,7 @@ class SizerBase(Sizer):
 
         # ALB moved this before Layout, it seems to be needed for wx2.6...
         self.notebook.Show()
+        self.notebook.SetSize(self.property_window.GetClientSize())
 
         self.property_window.Layout()
         self.property_window.SetTitle(_('Properties - <%s>') % self.name)
@@ -918,7 +977,7 @@ class SizerBase(Sizer):
             if not self.window.properties['size'].is_active():
                 szr = self.window.sizer.widget
                 w, h = self.window.widget.GetBestSize()
-                szr.SetItemMinSize(self.window.widget, w, h)            
+                szr.SetItemMinSize(self.window.widget, w, h)
             if self.window.sizer is not self:
                 self.window.sizer.layout(False)
             else:
@@ -926,6 +985,7 @@ class SizerBase(Sizer):
             return
         elif self.toplevel and isinstance(self.window, TopLevelBase):
             #self.window.widget.Layout()
+            self.widget.Layout()
             evt = wx.SizeEvent(self.window.widget.GetSize(),
                               self.window.widget.GetId())
             wx.PostEvent(self.window.widget, evt)
@@ -1099,7 +1159,13 @@ class SizerBase(Sizer):
                 try: c.item.widget.Refresh()
                 except AttributeError: pass
 
-    def update_view(self, *args): pass
+    def update_view(self, selected):
+        if self._btn is not None:
+            color = wx.SystemSettings_GetColour(wx.SYS_COLOUR_BTNFACE)
+            if selected:
+                color = wx.RED
+            self._btn.SetBackgroundColour(color)
+            self._btn.Refresh(True)
 
     def add_slot(self, *args, **kwds):
         """\
@@ -1190,7 +1256,6 @@ class SizerBase(Sizer):
         if not self.toplevel: return
         if not self.window.properties['size'].is_active():
             self.fit_parent()
-            import config
             w, h = self.widget.GetSize()
             prefix = ''
             if config.preferences.use_dialog_units:
@@ -1243,7 +1308,6 @@ class EditBoxSizer(SizerBase):
         self.widget.Add(self._btn, 0, wx.EXPAND)
         to_lay_out = []
         for c in self.children[1:]: # we've already added self._btn
-            #print c.item
             c.item.show_widget(True)
             if isinstance(c.item, SizerSlot):
                 self.widget.Add(c.item.widget, 1, wx.EXPAND)
@@ -1279,7 +1343,6 @@ class EditBoxSizer(SizerBase):
 ##                     w, h = c.item.widget.GetBestSize()
 ##                 self.widget.SetItemMinSize(c.item.widget, w, h)
         for pos, size in to_lay_out:
-            #print 'set_item:', pos, size
             self.set_item(pos, size=size, force_layout=False)
         self.layout(True)
         if not self.toplevel and getattr(self, 'sizer'):
@@ -1362,8 +1425,8 @@ class EditStaticBoxSizer(SizerBase):
                     w, h = c.item.widget.GetBestSize()
                 self.widget.SetItemMinSize(c.item.widget, w, h)
         self.layout()
-        if not self.toplevel and hasattr(self, 'sizer'):
-            # hasattr(self, 'sizer') is False only in case of a 'change_sizer'
+        if not self.toplevel and getattr(self, 'sizer'):
+            # getattr(self, 'sizer') is False only in case of a 'change_sizer'
             # call
             self.sizer.add_item(self, self.pos, self.option, self.flag,
                                 self.border, self.widget.GetMinSize())
@@ -1371,7 +1434,7 @@ class EditStaticBoxSizer(SizerBase):
     def _property_setup(self):
         SizerBase._property_setup(self)
         self.access_functions['label'] = (self.get_label, self.set_label)
-        lbl = self.properties['label'] = TextProperty(self, 'label', None)
+        lbl = self.properties['label'] = TextProperty(self, 'label', None, label=_("label"))
         def write(outfile, tabs):
             import widget_properties
             outfile.write('    ' * tabs + '<label>')
@@ -1469,7 +1532,7 @@ class CustomSizer(wx.BoxSizer):
         self._grid.SetItemMinSize(item, w, h)
 
     def GetChildren(self):
-        return [None] + self._grid.GetChildren()
+        return [None] + list(self._grid.GetChildren())
 
     def Layout(self):
         self._grid.Layout()
@@ -1555,22 +1618,16 @@ class GridSizerBase(SizerBase):
             self.set_item(pos, size=size, force_layout=False)
         self.layout(True)
         
-##         self.widget.Layout()
-
-##         print self.name, 'rows, cols:', self.rows, self.cols, len(self.children)
-##         self.set_rows(self.rows)
-##         self.set_cols(self.cols)
-
     def _property_setup(self):
         SizerBase._property_setup(self)
         self.access_functions['rows'] = (self.get_rows, self.set_rows)
         self.access_functions['cols'] = (self.get_cols, self.set_cols)
         self.access_functions['hgap'] = (self.get_hgap, self.set_hgap)
         self.access_functions['vgap'] = (self.get_vgap, self.set_vgap)
-        props = { 'rows': SpinProperty(self, 'rows', None),
-                  'cols': SpinProperty(self, 'cols', None),
-                  'hgap': SpinProperty(self, 'hgap', None),
-                  'vgap': SpinProperty(self, 'vgap', None) }
+        props = { 'rows': SpinProperty(self, 'rows', None, label=_("rows")),
+                  'cols': SpinProperty(self, 'cols', None, label=_("cols")),
+                  'hgap': SpinProperty(self, 'hgap', None, label=_("hgap")),
+                  'vgap': SpinProperty(self, 'vgap', None, label=_("vgap")) }
         self.properties = props
 
     def create_properties(self):
@@ -1589,7 +1646,7 @@ class GridSizerBase(SizerBase):
         page.SetAutoLayout(True)
         page.SetSizer(sizer)
         sizer.Fit(page)
-        self.notebook.AddPage(page, "Grid")
+        self.notebook.AddPage(page, _("Grid"))
 
     def get_rows(self): return self.rows
     def get_cols(self): return self.cols
@@ -1615,7 +1672,7 @@ class GridSizerBase(SizerBase):
     def set_cols(self, cols):
         self.cols = int(cols)
         if self.widget:
-            self.widget.SetCols(self.rows)
+            self.widget.SetCols(self.cols)
             self.layout(True)
 
     def set_hgap(self, hgap):
@@ -1777,8 +1834,8 @@ class EditGridSizer(GridSizerBase):
     def create_widget(self):
         self.widget = CustomSizer(self, wx.GridSizer, self.rows, self.cols,
                                   self.vgap, self.hgap)
-        if not self.toplevel and getattr(self, 'sizer', None): #hasattr(self, 'sizer'):
-            # hasattr(self, 'sizer') is False only in case of a 'change_sizer'
+        if not self.toplevel and getattr(self, 'sizer', None): 
+            # getattr(self, 'sizer') is False only in case of a 'change_sizer'
             # call
             self.sizer.add_item(self, self.pos, self.option, self.flag,
                                 self.border) #, self.widget.GetMinSize())
@@ -1801,16 +1858,17 @@ class CheckListDialogProperty(DialogProperty):
                     self.message = wx.StaticText(self, -1, "")
                     sizer.Add(self.message, 0,
                               wx.TOP|wx.LEFT|wx.RIGHT|wx.EXPAND, 10)
-                    self.choices = wx.CheckListBox(self, -1, choices=[])
+                    self.choices = wx.CheckListBox(self, -1, choices=['dummy'])
                     sizer.Add(self.choices, 1, wx.EXPAND|wx.LEFT|wx.RIGHT, 10)
                     sizer.Add(wx.StaticLine(self, -1), 0, wx.EXPAND|wx.ALL, 10)
                     sz2 = wx.BoxSizer(wx.HORIZONTAL)
-                    sz2.Add(wx.Button(self, wx.ID_OK, _("OK")), 0, wx.ALL, 10)
-                    sz2.Add(wx.Button(self, wx.ID_CANCEL, _("Cancel")), 0,wx.ALL, 10)
+                    sz2.Add(wx.Button(self, wx.ID_OK, ""), 0, wx.ALL, 10)
+                    sz2.Add(wx.Button(self, wx.ID_CANCEL, ""), 0, wx.ALL, 10)
                     sizer.Add(sz2, 0, wx.ALIGN_CENTER)
                     self.SetAutoLayout(True)
                     self.SetSizer(sizer)
                     sizer.Fit(self)
+                    self.CenterOnScreen()
 
                 def get_value(self):
                     ret = []
@@ -1834,7 +1892,7 @@ class CheckListDialogProperty(DialogProperty):
             self.dialog[0] = Dialog()
             
         DialogProperty.__init__(self, owner, name, parent, self.dialog[0],
-                                can_disable)
+                                can_disable, label=title)
         self.choices_setter = callback
 
     def display_dialog(self, event):
@@ -1882,7 +1940,8 @@ class EditFlexGridSizer(GridSizerBase):
         def rows_setter():
             return map(str, range(self.get_rows()))
         pr = CheckListDialogProperty(self, 'growable_rows', None,
-                                     _('Growable Rows'), _('Select growable rows'),
+                                     _('Growable Rows'),
+                                     _('Select growable rows'),
                                      rows_setter)
         self.properties['growable_rows'] = pr
         def cols_setter():
@@ -1998,9 +2057,10 @@ def builder(parent, sizer, pos, number=[1], show=True):
     class SizerDialog(wx.Dialog):
         def __init__(self, parent):
             wx.Dialog.__init__(self, misc.get_toplevel_parent(parent), -1,
-                              'Select sizer type')
+                              _('Select sizer type'))
             self.orientation = wx.RadioBox(self, -1, _('Orientation'),
-                                          choices=[_('Horizontal'), _('Vertical')])
+                                          choices=[_('Horizontal'),
+                                                   _('Vertical')])
             self.orientation.SetSelection(0)
             tmp = wx.BoxSizer(wx.HORIZONTAL)
             tmp.Add(wx.StaticText(self, -1, _('Slots: ')), 0,
@@ -2030,7 +2090,7 @@ def builder(parent, sizer, pos, number=[1], show=True):
             self.SetSizer(szr)
             szr.Fit(self)
             self.Layout()
-            self.CentreOnParent()
+            self.CenterOnScreen()
 
         def reset(self):
             self.orientation.SetSelection(0)
@@ -2063,7 +2123,7 @@ def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     """
     from xml_parse import XmlParsingError
     try: name = attrs['name']
-    except KeyError: raise XmlParsingError, "'name' attribute missing"
+    except KeyError: raise XmlParsingError, _("'name' attribute missing")
     orientation = wx.VERTICAL # default value
     if sizer is not None: topl = False
     else: topl = True
@@ -2073,7 +2133,7 @@ def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
         sz = EditBoxSizer(name, parent, orientation, 0, topl)
     if sizer is not None:
         if sizeritem is None:
-            raise XmlParsingError, "'sizeritem' object not found"
+            raise XmlParsingError, _("'sizeritem' object not found")
         sizer.add_item(sz, pos=pos, option=sizeritem.option,
                        flag=sizeritem.flag, border=sizeritem.border) 
         node = Tree.Node(sz)
@@ -2096,10 +2156,10 @@ def grid_builder(parent, sizer, pos, number=[1], show=True):
         def __init__(self, parent):
             wx.Dialog.__init__(self, misc.get_toplevel_parent(parent), -1,
                               _('Select sizer attributes'))
-            self.rows = SpinProperty(self, 'rows', self)
-            self.cols = SpinProperty(self, 'cols', self)
-            self.vgap = SpinProperty(self, 'vgap', self)
-            self.hgap = SpinProperty(self, 'hgap', self)
+            self.rows = SpinProperty(self, 'rows', self, label=_("rows"))
+            self.cols = SpinProperty(self, 'cols', self, label=_("cols"))
+            self.vgap = SpinProperty(self, 'vgap', self, label=_("vgap"))
+            self.hgap = SpinProperty(self, 'hgap', self, label=_("hgap"))
             self.flex = wx.CheckBox(self, -1, '')
 
             self.rows.set_value(3)
@@ -2108,7 +2168,7 @@ def grid_builder(parent, sizer, pos, number=[1], show=True):
             self.hgap.set_value(0)
 
             szr = wx.BoxSizer(wx.HORIZONTAL)
-            szr.Add(wx.StaticText(self, -1, 'Flexible'), 2,
+            szr.Add(wx.StaticText(self, -1, _('Flexible')), 2,
                     wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
             szr.Add(self.flex, 5, wx.ALL, 4)
 
@@ -2179,13 +2239,13 @@ def grid_xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     """
     from xml_parse import XmlParsingError
     try: name = attrs['name']
-    except KeyError: raise XmlParsingError, "'name' attribute missing"
+    except KeyError: raise XmlParsingError, _("'name' attribute missing")
     if attrs['base'] == 'EditGridSizer': constructor = EditGridSizer
     else: constructor = EditFlexGridSizer
     if sizer is not None: 
         sz = constructor(name, parent, rows=0, cols=0, toplevel=False)
         if sizeritem is None:
-            raise XmlParsingError, "'sizeritem' object not found"
+            raise XmlParsingError, _("'sizeritem' object not found")
         sizer.add_item(sz, pos=pos, option=sizeritem.option,
                        flag=sizeritem.flag, border=sizeritem.border)
         node = Tree.Node(sz)
@@ -2216,7 +2276,6 @@ def init_all():
     cwx['EditGridSizer'] = grid_xml_builder
     cwx['EditFlexGridSizer'] = grid_xml_builder
 
-    from tree import WidgetTree
     import os.path
     WidgetTree.images['EditStaticBoxSizer'] = os.path.join(common.wxglade_path,
                                                            'icons/sizer.xpm')
@@ -2225,6 +2284,3 @@ def init_all():
 
     return [common.make_object_button('EditBoxSizer', 'icons/sizer.xpm'),
             common.make_object_button('EditGridSizer', 'icons/grid_sizer.xpm')]
-
-
-
