@@ -1,5 +1,5 @@
 # edit_windows.py: base classes for windows used by wxGlade
-# $Id: edit_windows.py,v 1.88 2007/03/27 07:02:07 agriggio Exp $
+# $Id: edit_windows.py,v 1.90 2007/08/07 12:21:56 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
@@ -7,8 +7,7 @@
 
 import wx
 from widget_properties import *
-from tree import Tree, WidgetTree
-import math, misc, common, sys, config
+import math, misc, common, config
 import re
 
 # ALB 2004-12-05: event handling support
@@ -33,6 +32,7 @@ class EditBase(EventsMixin):
         # contained here, but in a separate list (see ManagedBase)
         # the keys of the dict are the names of the properties
         self.properties = {}
+        self.property_blocking = {}
         self.parent = parent
         # id used for internal purpose events
         self.id = id
@@ -50,9 +50,9 @@ class EditBase(EventsMixin):
 
         # these two properties are special and are not listed in
         # 'self.properties'
-        self.name_prop = TextProperty(self, 'name', None)
+        self.name_prop = TextProperty(self, 'name', None, label=_("name"))
         self.klass_prop = TextProperty(self, 'class', None,
-                                       readonly=not custom_class)
+                                       readonly=not custom_class, label=_("class"))
         if custom_class:
             self.klass_prop.tooltip = _("If you change the default value, " \
                                       "it will be interpreted as the name " \
@@ -61,6 +61,23 @@ class EditBase(EventsMixin):
                                       "depends on the kind (i.e. language) " \
                                       "of output. See the docs for " \
                                       "more details.")
+
+        # ALB 2007-08-31: custom base classes support
+        if getattr(self, '_custom_base_classes', False):
+            self.custom_base = ""
+            def get_custom_base(): return self.custom_base
+            def set_custom_base(val): self.custom_base = val
+            self.access_functions['custom_base'] = (get_custom_base,
+                                                    set_custom_base)
+            p = self.properties['custom_base'] = TextProperty(
+                self, 'custom_base', can_disable=True, enabled=False)
+            p.label = _('Base class(es)')
+            p.tooltip = _("""\
+A comma-separated list of custom base classes. The first will be invoked \
+with the same parameters as this class, while for the others the default \
+constructor will be used. You should probably not use this if \
+"overwrite existing sources" is not set.""")
+            
         self.notebook = None
         self.property_window = property_window
 
@@ -79,6 +96,11 @@ class EditBase(EventsMixin):
         # ALB 2004-12-05
         EventsMixin.__init__(self)
 
+        # code property
+        import code_property
+        self.properties['extracode'] = code_property.CodeProperty(self)
+        self.properties['extraproperties'] = code_property.ExtraPropertiesProperty(self)
+
     def show_widget(self, yes):
         if yes and self.widget is None:
             self.create_widget()
@@ -131,11 +153,13 @@ class EditBase(EventsMixin):
         self.notebook.SetAutoLayout(True)
         self.notebook.Hide()
 
-        self._common_panel = panel = wx.ScrolledWindow(self.notebook, -1,
-                                                      style=wx.TAB_TRAVERSAL)
+        self._common_panel = panel = wx.ScrolledWindow(
+            self.notebook, -1, style=wx.TAB_TRAVERSAL|wx.FULL_REPAINT_ON_RESIZE)
 
         self.name_prop.display(panel)
         self.klass_prop.display(panel)
+        if getattr(self, '_custom_base_classes', False):
+            self.properties['custom_base'].display(panel)
 
     def __getitem__(self, value):
         return self.access_functions[value]
@@ -177,23 +201,42 @@ class EditBase(EventsMixin):
                 COPY_ID, REMOVE_ID, CUT_ID = [wx.NewId() for i in range(3)]
                 self._rmenu = misc.wxGladePopupMenu(self.name)
                 misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
-                                 'remove.xpm')
+                                 wx.ART_DELETE)
                 misc.append_item(self._rmenu, COPY_ID, _('Copy\tCtrl+C'),
-                                 'copy.xpm')
+                                 wx.ART_COPY)
                 misc.append_item(self._rmenu, CUT_ID, _('Cut\tCtrl+X'),
-                                 'cut.xpm')
+                                 wx.ART_CUT)
+                self._rmenu.AppendSeparator()
+                PREVIEW_ID = wx.NewId()
+                misc.append_item(self._rmenu, PREVIEW_ID, _('Preview'))
                 def bind(method):
                     return lambda e: misc.wxCallAfter(method)
                 wx.EVT_MENU(self.widget, REMOVE_ID, bind(self.remove))
                 wx.EVT_MENU(self.widget, COPY_ID, bind(self.clipboard_copy))
                 wx.EVT_MENU(self.widget, CUT_ID, bind(self.clipboard_cut))
-                
+                wx.EVT_MENU(self.widget, PREVIEW_ID, bind(self.preview_parent))
+
+            self.setup_preview_menu()
             self.widget.PopupMenu(self._rmenu, event.GetPosition())
 
     def remove(self, *args):
         self._dont_destroy = False # always destroy when explicitly asked
         common.app_tree.remove(self.node)
 
+    def setup_preview_menu(self):
+        p = misc.get_toplevel_widget(self)
+        if p is not None:
+            item = list(self._rmenu.GetMenuItems())[-1]
+            if p.preview_is_visible():
+                item.SetText(_('Close preview') + ' (%s)\tCtrl+P' % p.name)
+            else:
+                item.SetText(_('Preview') + ' (%s)\tCtrl+P' % p.name)        
+
+    def preview_parent(self):
+        widget = misc.get_toplevel_widget(self)
+        if widget is not None:
+            widget.preview(None)
+
     def show_properties(self, *args):
         """\
         Updates property_window to display the properties of self
@@ -206,23 +249,30 @@ class EditBase(EventsMixin):
                 nb = self.parent
                 if nb.widget:
                     i = 0
-                    for tn,ep in nb.tabs: # tn=tabname, ep = editpanel 
-                        if self.name == ep.name:      # If I am under this tab...
-                            nb.widget.SetSelection(i) # ...Show that tab.
+                    for tn, ep in nb.tabs: # tn=tabname, ep = editpanel
+                        try:
+                            if ep and self.name == ep.name:
+                                # If I am under this tab...
+                                nb.widget.SetSelection(i) # ...Show that tab.
+                        except AttributeError:
+                            pass
                         i = i + 1
-        if self.parent and self.parent.klass == 'wxPanel': # am I a widget under a wxPanel under a wxNotebook?
+        if self.parent and self.parent.klass == 'wxPanel':
+            # am I a widget under a wxPanel under a wxNotebook?
             if self.parent.parent and self.parent.parent.klass == 'wxNotebook':
                 #pdb.set_trace()
                 nb = self.parent.parent
                 if nb.widget:
                     i = 0
-                    for tn,ep in nb.tabs: # tn=tabname, ep = editpanel 
-                        if self.parent.name == ep.name:
-                            nb.widget.SetSelection(i)
+                    for tn, ep in nb.tabs: # tn=tabname, ep = editpanel
+                        try:
+                            if ep and self.parent.name == ep.name:
+                                nb.widget.SetSelection(i)
+                        except AttributeError:
+                            pass
                         i = i + 1
         # End Marcello 13 oct. 2005
 
-
         if not self.is_visible(): return # don't do anything if self is hidden
         # create the notebook the first time the function is called: this
         # allows us to create only the notebooks we really need
@@ -230,6 +280,7 @@ class EditBase(EventsMixin):
             self.create_properties()
             # ALB 2004-12-05
             self.create_events_property()
+            self.create_extracode_property()
         sizer_tmp = self.property_window.GetSizer()
         #sizer_tmp = wxPyTypeCast(sizer_tmp, "wxBoxSizer")
         #child = wxPyTypeCast(sizer_tmp.GetChildren()[0], "wxSizerItem")
@@ -256,6 +307,7 @@ class EditBase(EventsMixin):
 
         # ALB moved this before Layout, it seems to be needed for wx2.6...
         self.notebook.Show()
+        self.notebook.SetSize(self.property_window.GetClientSize())
 
         self.property_window.Layout()
         self.property_window.SetTitle(_('Properties - <%s>') % self.name)
@@ -318,6 +370,33 @@ class EditBase(EventsMixin):
         """
         pass
 
+
+    def create_extracode_property(self):
+        try:
+            self.properties['extracode']._show(self.notebook)
+            self.properties['extraproperties']._show(self.notebook)
+        except KeyError:
+            pass
+
+    def set_property_blocking(self, key, item):
+        if self.property_blocking.has_key(key):
+            self.property_blocking[key].append(item)
+        else:
+            self.property_blocking[key] = [item]
+
+    def get_property_blocking(self, key):
+        if self.property_blocking.has_key(key):
+            return self.property_blocking[key]
+        return None
+
+    def remove_property_blocking(self, key, item):
+        if self.property_blocking.has_key(key):
+            for i in range(self.property_blocking[key].count(item)):
+                self.property_blocking[key].remove(item)
+            if not len(self.property_blocking[key]):
+                del self.property_blocking[key]
+
+
 # end of class EditBase
 
 
@@ -365,27 +444,29 @@ class WindowBase(EditBase):
 
         prop = self.properties
         prop['id'] = TextProperty(self, 'id', None, can_disable=True)
-        prop['size'] = TextProperty(self, 'size', None, can_disable=True)
-        prop['background'] = ColorDialogProperty(self, "background", None)
-        prop['foreground'] = ColorDialogProperty(self, "foreground", None)
-        prop['font'] = FontDialogProperty(self, "font", None)
+        prop['size'] = TextProperty(self, 'size', None, can_disable=True, label=_("size"))
+        prop['background'] = ColorDialogProperty(self, "background", None, label=_("background"))
+        prop['foreground'] = ColorDialogProperty(self, "foreground", None, label=_("foreground"))
+        prop['font'] = FontDialogProperty(self, "font", None, label=_("font"))
 
         # properties added 2002-08-15
-        prop['tooltip'] = TextProperty(self, 'tooltip', None, can_disable=True)
+        prop['tooltip'] = TextProperty(self, 'tooltip', None, can_disable=True,  label=_('tooltip'))
 
         # properties added 2003-05-15
         self.disabled_p = False
         self.access_functions['disabled'] = (self.get_disabled,
                                              self.set_disabled)
-        prop['disabled'] = CheckBoxProperty(self, 'disabled', None)
+        prop['disabled'] = CheckBoxProperty(self, 'disabled', None, _('disabled'))
         
         self.focused_p = False
         self.access_functions['focused'] = (self.get_focused, self.set_focused)
-        prop['focused'] = CheckBoxProperty(self, 'focused', None)
+        prop['focused'] = CheckBoxProperty(self, 'focused', None, _('focused'))
 
         self.hidden_p = False
         self.access_functions['hidden'] = (self.get_hidden, self.set_hidden)
-        prop['hidden'] = CheckBoxProperty(self, 'hidden', None)
+        prop['hidden'] = CheckBoxProperty(self, 'hidden', None, _('hidden'))
+
+        
 
     def finish_widget_creation(self, *args, **kwds):
         self._original['background'] = self.widget.GetBackgroundColour()
@@ -426,11 +507,14 @@ class WindowBase(EditBase):
             evt_flags = 0
             if event.ControlDown(): evt_flags = wx.ACCEL_CTRL
             evt_key = event.GetKeyCode()
+            done = False
             for flags, key, function in misc.accel_table:
                 if evt_flags == flags and evt_key == key:
                     misc.wxCallAfter(function)
+                    done = True
                     break
-            event.Skip()
+            if not done:
+                event.Skip()
         wx.EVT_KEY_DOWN(self.widget, on_key_down)
 
     def create_properties(self):
@@ -460,6 +544,8 @@ class WindowBase(EditBase):
         sizer_tmp = wx.BoxSizer(wx.VERTICAL)
         sizer_tmp.Add(self.name_prop.panel, 0, wx.EXPAND)
         sizer_tmp.Add(self.klass_prop.panel, 0, wx.EXPAND)
+        if getattr(self, '_custom_base_classes', False):
+            sizer_tmp.Add(prop['custom_base'].panel, 0, wx.EXPAND)
         sizer_tmp.Add(prop['id'].panel, 0, wx.EXPAND)
         sizer_tmp.Add(prop['size'].panel, 0, wx.EXPAND)
         sizer_tmp.Add(prop['background'].panel, 0, wx.EXPAND)
@@ -482,6 +568,7 @@ class WindowBase(EditBase):
         panel.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
 
 
+
     def on_size(self, event):
         """\
         update the value of the 'size' property
@@ -658,6 +745,9 @@ class WindowBase(EditBase):
                     self.props[self.index] = str(data.strip())
             # end of class FontHandler
             return FontHandler(self)
+        elif name == 'extraproperties':
+            import code_property
+            return code_property.ExtraPropertiesPropertyHandler(self)
         return EditBase.get_property_handler(self, name)
 
     def get_disabled(self):
@@ -718,11 +808,11 @@ class ManagedBase(WindowBase):
                           wx.EXPAND, wx.ALIGN_RIGHT, wx.ALIGN_BOTTOM,
                           wx.ALIGN_CENTER_HORIZONTAL, wx.ALIGN_CENTER_VERTICAL,
                           wx.SHAPED, wx.ADJUST_MINSIZE)
-        flag_labels = ('#section#Border',
+        flag_labels = (u'#section#' + _('Border'),
                        'wxALL',
                        'wxLEFT', 'wxRIGHT',
                        'wxTOP', 'wxBOTTOM',
-                       '#section#Alignment', 'wxEXPAND', 'wxALIGN_RIGHT',
+                       u'#section#' + _('Alignment'), 'wxEXPAND', 'wxALIGN_RIGHT',
                        'wxALIGN_BOTTOM', 'wxALIGN_CENTER_HORIZONTAL',
                        'wxALIGN_CENTER_VERTICAL', 'wxSHAPED',
                        'wxADJUST_MINSIZE')
@@ -734,13 +824,13 @@ class ManagedBase(WindowBase):
         sizer.add_item(self, pos)
 
         szprop = self.sizer_properties
-        #szprop['option'] = SpinProperty(self, "option", None, 0, (0, 1000))
+        #szprop['option'] = SpinProperty(self, _("option"), None, 0, (0, 1000))
         from layout_option_property import LayoutOptionProperty, \
              LayoutPosProperty
         szprop['option'] = LayoutOptionProperty(self, sizer)
         
         szprop['flag'] = CheckListProperty(self, 'flag', None, flag_labels)
-        szprop['border'] = SpinProperty(self, 'border', None, 0, (0, 1000))
+        szprop['border'] = SpinProperty(self, 'border', None, 0, (0, 1000), label=_('border'))
 ##         pos_p = szprop['pos'] = SpinProperty(self, 'pos', None, 0, (0, 1000))
 ##         def write(*args, **kwds): pass
 ##         pos_p.write = write # no need to save the position
@@ -767,7 +857,8 @@ class ManagedBase(WindowBase):
     def create_properties(self):
         WindowBase.create_properties(self)
         if not self._has_layout: return
-        panel = wx.ScrolledWindow(self.notebook, -1, style=wx.TAB_TRAVERSAL)
+        panel = wx.ScrolledWindow(
+            self.notebook, -1, style=wx.TAB_TRAVERSAL|wx.FULL_REPAINT_ON_RESIZE)
 
         min_x = wx.SystemSettings_GetMetric(wx.SYS_WINDOWMIN_X)
         min_y = wx.SystemSettings_GetMetric(wx.SYS_WINDOWMIN_Y)
@@ -775,10 +866,10 @@ class ManagedBase(WindowBase):
         max_y = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)
 
         szprop = self.sizer_properties
-        szprop['option'].display(panel)
-        szprop['flag'].display(panel)
-        szprop['border'].display(panel)
         szprop['pos'].display(panel)
+        szprop['option'].display(panel)
+        szprop['border'].display(panel)
+        szprop['flag'].display(panel)
 
         sizer_tmp = wx.BoxSizer(wx.VERTICAL)
         sizer_tmp.Add(szprop['pos'].panel, 0, wx.EXPAND)
@@ -791,7 +882,7 @@ class ManagedBase(WindowBase):
         sizer_tmp.Fit(panel)
 
         w, h = panel.GetClientSize()
-        self.notebook.AddPage(panel, "Layout")
+        self.notebook.AddPage(panel, _("Layout"))
         panel.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
         
     def update_view(self, selected):
@@ -950,7 +1041,6 @@ class PreviewMixin:
         sizer_tmp.Fit(panel)
         w, h = panel.GetClientSize()
         self.property_window.Layout()
-        import math
         panel.SetScrollbars(1, 5, 1, int(math.ceil(h/5.0)))
 
     def preview(self, event):
@@ -963,6 +1053,9 @@ class PreviewMixin:
             # (see application.py -> preview)
             self.preview_widget.Close()
 
+    def preview_is_visible(self):
+        return self.preview_widget is not None
+
 # end of class PreviewMixin
 
 
@@ -971,6 +1064,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
     Base class for every non-managed window (i.e. Frames and Dialogs).
     """
     _is_toplevel = True
+    _custom_base_classes = True
     
     def __init__(self, name, klass, parent, id, property_window, show=True,
                  has_title=True, title=None):
@@ -981,7 +1075,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
             if title is None: title = self.name
             self.title = title
             self.access_functions['title'] = (self.get_title, self.set_title)
-            self.properties['title'] = TextProperty(self, 'title', None)
+            self.properties['title'] = TextProperty(self, 'title', None, label=_("title"))
         self.sizer = None # sizer that controls the layout of the children
                           # of the window
         PreviewMixin.__init__(self)
@@ -1022,7 +1116,7 @@ class TopLevelBase(WindowBase, PreviewMixin):
                 REMOVE_ID, HIDE_ID = [wx.NewId() for i in range(2)]
                 self._rmenu = misc.wxGladePopupMenu(self.name)
                 misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
-                                 'remove.xpm')
+                                 wx.ART_DELETE)
                 misc.append_item(self._rmenu, HIDE_ID, _('Hide'))
                 def bind(method):
                     return lambda e: misc.wxCallAfter(method)
@@ -1031,9 +1125,14 @@ class TopLevelBase(WindowBase, PreviewMixin):
                 # paste
                 PASTE_ID = wx.NewId()
                 misc.append_item(self._rmenu, PASTE_ID, _('Paste\tCtrl+V'),
-                                 'paste.xpm')
+                                 wx.ART_PASTE)
                 wx.EVT_MENU(self.widget, PASTE_ID, bind(self.clipboard_paste))
-                
+                PREVIEW_ID = wx.NewId()
+                self._rmenu.AppendSeparator()
+                misc.append_item(self._rmenu, PREVIEW_ID, _('Preview'))
+                wx.EVT_MENU(self.widget, PREVIEW_ID, bind(self.preview_parent))
+
+            self.setup_preview_menu()
             self.widget.PopupMenu(self._rmenu, event.GetPosition())
 
     def clipboard_paste(self, *args):
