@@ -1,23 +1,25 @@
 # main.py: Main wxGlade module: defines wxGladeFrame which contains the buttons
 # to add widgets and initializes all the stuff (tree, property_frame, etc.)
-# $Id: main.py,v 1.83 2007/03/27 07:03:35 agriggio Exp $
+# $Id: main.py,v 1.84 2007/08/07 12:21:56 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
-import wxversion
-wxversion.ensureMinimal("2.6")
+import sys
+
+# ensure minimal wx version
+if not hasattr(sys, 'frozen'):
+    import wxversion
+    wxversion.ensureMinimal("2.6")
 
-#from wxPython.wx import *
 import wx
 from widget_properties import *
-from tree import Tree, WidgetTree
-import edit_sizers
+from tree import WidgetTree
 import common, os, os.path, misc, config
 import clipboard
 
-import xml_parse
+import template
 
 
 class wxGladePropertyPanel(wx.Panel):
@@ -161,30 +163,33 @@ class wxGladeFrame(wx.Frame):
         core_btns, custom_btns = common.load_widgets()
         sizer_btns = common.load_sizers()
         
+        append_item = misc.append_item
         self.TREE_ID = TREE_ID = wx.NewId()
-        view_menu.Append(TREE_ID, _("Show &Tree\tCtrl+T"), "", True)
-        view_menu.Check(TREE_ID, True)
+        append_item(view_menu, TREE_ID, _("Show &Tree\tF2"))
         self.PROPS_ID = PROPS_ID = wx.NewId()
         self.RAISE_ID = RAISE_ID = wx.NewId()
-        view_menu.Append(PROPS_ID, _("Show &Properties\tCtrl+P"), "", True)
-        view_menu.Check(PROPS_ID, True)
-        view_menu.Append(RAISE_ID, _("&Raise All\tCtrl-R"), "", False)
-        append_item = misc.append_item
+        append_item(view_menu, PROPS_ID, _("Show &Properties\tF3"))
+        append_item(view_menu, RAISE_ID, _("&Raise All\tF4"))
         NEW_ID = wx.NewId()
-        append_item(file_menu, NEW_ID, _("&New\tCtrl+N"), 'new.xpm')
+        append_item(file_menu, NEW_ID, _("&New\tCtrl+N"), wx.ART_NEW)
+        NEW_FROM_TEMPLATE_ID = wx.NewId()
+        append_item(file_menu, NEW_FROM_TEMPLATE_ID,
+                    _("New from &Template...\tShift+Ctrl+N"))
         OPEN_ID = wx.NewId()
-        append_item(file_menu, OPEN_ID, _("&Open...\tCtrl+O"), 'open.xpm') 
+        append_item(file_menu, OPEN_ID, _("&Open...\tCtrl+O"), wx.ART_FILE_OPEN)
         SAVE_ID = wx.NewId()
-        append_item(file_menu, SAVE_ID, _("&Save\tCtrl+S"), 'save.xpm') 
+        append_item(file_menu, SAVE_ID, _("&Save\tCtrl+S"), wx.ART_FILE_SAVE)
         SAVE_AS_ID = wx.NewId()
         append_item(file_menu, SAVE_AS_ID, _("Save As...\tShift+Ctrl+S"),
-                    'save_as.xpm')
+                    wx.ART_FILE_SAVE_AS)
+        SAVE_TEMPLATE_ID = wx.NewId()
+        append_item(file_menu, SAVE_TEMPLATE_ID, _("Save As Template..."))
         file_menu.AppendSeparator()
-        RELOAD_ID = wx.NewId()
-        append_item(file_menu, RELOAD_ID, _("&Refresh\tf5"), 'refresh.xpm')
+        RELOAD_ID = wx.ID_REFRESH #wx.NewId()
+        append_item(file_menu, RELOAD_ID, _("&Refresh\tf5")) #, wx.ART_REDO)
         GENERATE_CODE_ID = wx.NewId()
         append_item(file_menu, GENERATE_CODE_ID, _("&Generate Code\tCtrl+G"),
-                    'generate.xpm')
+                    wx.ART_EXECUTABLE_FILE)
         
         file_menu.AppendSeparator()
         IMPORT_ID = wx.NewId()
@@ -192,16 +197,20 @@ class wxGladeFrame(wx.Frame):
         
         EXIT_ID = wx.NewId()
         file_menu.AppendSeparator()
-        append_item(file_menu, EXIT_ID, _('E&xit\tCtrl+Q'), 'exit.xpm')
-        PREFS_ID = wx.NewId()
+        append_item(file_menu, EXIT_ID, _('E&xit\tCtrl+Q'), wx.ART_QUIT)
+        PREFS_ID = wx.ID_PREFERENCES #NewId()
         view_menu.AppendSeparator()
-        append_item(view_menu, PREFS_ID, _('Preferences...'), 'prefs.xpm')
+        MANAGE_TEMPLATES_ID = wx.NewId()
+        append_item(view_menu, MANAGE_TEMPLATES_ID, _('Templates Manager...'))
+        view_menu.AppendSeparator()
+        append_item(view_menu, PREFS_ID, _('Preferences...'))
+        #wx.ART_HELP_SETTINGS)
         menu_bar.Append(file_menu, _("&File"))
         menu_bar.Append(view_menu, _("&View"))
         TUT_ID = wx.NewId()
-        append_item(help_menu, TUT_ID, _('Contents\tF1'), 'tutorial.xpm')
-        ABOUT_ID = wx.NewId()
-        append_item(help_menu, ABOUT_ID, _('About...'), 'about.xpm')
+        append_item(help_menu, TUT_ID, _('Contents\tF1'), wx.ART_HELP)
+        ABOUT_ID = wx.ID_ABOUT #wx.NewId()
+        append_item(help_menu, ABOUT_ID, _('About...'))#, wx.ART_QUESTION)
         menu_bar.Append(help_menu, _('&Help'))
         parent.SetMenuBar(menu_bar)
         # Mac tweaks...
@@ -242,9 +251,11 @@ class wxGladeFrame(wx.Frame):
         wx.EVT_MENU(self, PROPS_ID, self.show_props_window)
         wx.EVT_MENU(self, RAISE_ID, self.raise_all)
         wx.EVT_MENU(self, NEW_ID, self.new_app)
+        wx.EVT_MENU(self, NEW_FROM_TEMPLATE_ID, self.new_app_from_template)
         wx.EVT_MENU(self, OPEN_ID, self.open_app)
         wx.EVT_MENU(self, SAVE_ID, self.save_app)
         wx.EVT_MENU(self, SAVE_AS_ID, self.save_app_as)
+        wx.EVT_MENU(self, SAVE_TEMPLATE_ID, self.save_app_as_template)
         def generate_code(event):
             common.app_tree.app.generate_code()
         wx.EVT_MENU(self, GENERATE_CODE_ID, generate_code)
@@ -252,9 +263,18 @@ class wxGladeFrame(wx.Frame):
         wx.EVT_MENU(self, TUT_ID, self.show_tutorial)
         wx.EVT_MENU(self, ABOUT_ID, self.show_about_box)
         wx.EVT_MENU(self, PREFS_ID, self.edit_preferences)
+        wx.EVT_MENU(self, MANAGE_TEMPLATES_ID, self.manage_templates)
         wx.EVT_MENU(self, IMPORT_ID, self.import_xrc)
         wx.EVT_MENU(self, RELOAD_ID, self.reload_app)
 
+        PREVIEW_ID = wx.NewId()
+        def preview(event):
+            if common.app_tree.cur_widget is not None:
+                p = misc.get_toplevel_widget(common.app_tree.cur_widget)
+                if p is not None:
+                    p.preview(None)
+        wx.EVT_MENU(self, PREVIEW_ID, preview)
+
         self.accel_table = wx.AcceleratorTable([
             (wx.ACCEL_CTRL, ord('N'), NEW_ID),
             (wx.ACCEL_CTRL, ord('O'), OPEN_ID),
@@ -265,6 +285,10 @@ class wxGladeFrame(wx.Frame):
             (0, wx.WXK_F1, TUT_ID),
             (wx.ACCEL_CTRL, ord('Q'), EXIT_ID),
             (0, wx.WXK_F5, RELOAD_ID),
+            (0, wx.WXK_F2, TREE_ID),
+            (0, wx.WXK_F3, PROPS_ID),
+            (0, wx.WXK_F4, RAISE_ID),
+            (wx.ACCEL_CTRL, ord('P'), PREVIEW_ID),
             ])
 
         # Tutorial window
@@ -344,8 +368,10 @@ class wxGladeFrame(wx.Frame):
         # Properties window
         frame_style = wx.DEFAULT_FRAME_STYLE
         frame_tool_win = config.preferences.frame_tool_win
-        if wx.Platform == '__WXMSW__' and frame_tool_win:
-            frame_style |= wx.FRAME_TOOL_WINDOW|wx.FRAME_NO_TASKBAR
+        if frame_tool_win:
+            frame_style |= wx.FRAME_NO_TASKBAR | wx.FRAME_FLOAT_ON_PARENT
+            frame_style &= ~wx.MINIMIZE_BOX
+            if wx.Platform != '__WXGTK__': frame_style |= wx.FRAME_TOOL_WINDOW
         
         self.frame2 = wx.Frame(self, -1, _('Properties - <app>'),
                               style=frame_style)
@@ -374,7 +400,7 @@ class wxGladeFrame(wx.Frame):
         self.frame2.SetSizer(sizer_tmp)
         sizer_tmp = wx.BoxSizer(wx.VERTICAL)
         def hide_frame2(event):
-            menu_bar.Check(PROPS_ID, False)
+            #menu_bar.Check(PROPS_ID, False)
             self.frame2.Hide()
         wx.EVT_CLOSE(self.frame2, hide_frame2)
         wx.EVT_CLOSE(self, self.cleanup)
@@ -395,7 +421,7 @@ class wxGladeFrame(wx.Frame):
         sizer_tmp.Fit(property_panel)
         
         def on_tree_frame_close(event):
-            menu_bar.Check(TREE_ID, False)
+            #menu_bar.Check(TREE_ID, False)
             self.tree_frame.Hide()
         wx.EVT_CLOSE(self.tree_frame, on_tree_frame_close)
         # check to see if there are some remembered values
@@ -426,23 +452,25 @@ class wxGladeFrame(wx.Frame):
                 w += 10
             self.frame2.SetPosition((x, y+h))
             self.tree_frame.SetPosition((x+w, y))
+        self.Show()
         self.tree_frame.Show()
         self.frame2.Show()    
-        self.Show()
 
-        self._skip_activate = False
+        #self._skip_activate = False
+##         if frame_tool_win:
+##             def on_iconize(event):
+##                 if event.Iconized():
+##                     self.hide_all()
+##                 else:
+##                     self.show_and_raise()
+##                 event.Skip()
+##             wx.EVT_ICONIZE(self, on_iconize)
+
         if wx.Platform == '__WXMSW__':
             import about
             # I'll pay a beer to anyone who can explain to me why this prevents
             # a segfault on Win32 when you exit without doing anything!!
             self.about_box = about.wxGladeAboutBox(self.GetParent())
-            def on_iconize(event):
-                if event.Iconized():
-                    self.hide_all()
-                else:
-                    self.show_and_raise()
-                event.Skip()
-            wx.EVT_ICONIZE(self, on_iconize)
         else:
             self.about_box = None
 
@@ -497,15 +525,18 @@ class wxGladeFrame(wx.Frame):
         config.edit_preferences()
 
     def show_tree(self, event):
-        self.tree_frame.Show(event.IsChecked())
+        self.tree_frame.Show()
+        self.tree_frame.Raise()
+        common.app_tree.SetFocus()
 
     def show_props_window(self, event):
-        show = event.IsChecked()
-        self.frame2.Show(show)
-        if show:
-            common.app_tree.app.show_properties()
-            if common.app_tree.cur_widget:
-                common.app_tree.cur_widget.show_properties()
+        self.frame2.Show()
+        self.frame2.Raise()
+        try:
+            c = self.frame2.GetSizer().GetChildren()
+            if c: c[0].GetWindow().SetFocus()
+        except (AttributeError, TypeError):
+            self.frame2.SetFocus()
 
     def raise_all(self, event):
         children = self.GetChildren()
@@ -554,14 +585,24 @@ class wxGladeFrame(wx.Frame):
             if config.preferences.autosave and self.autosave_timer is not None:
                 self.autosave_timer.Start()
 
+    def new_app_from_template(self, event):
+        """\
+        creates a new wxGlade project from an existing template file
+        """
+        if not self.ask_save(): return
+        infile = template.select_template()
+        if infile:
+            self._open_app(infile, add_to_history=False)
+            common.app_tree.app.template_data = None
+
     def reload_app(self, event):
         self.ask_save()
         if not common.app_tree.app.filename:
             wx.MessageBox(_("Impossible to reload an unsaved application"),
-                         _("Alert"), style=wx.OK|wx.ICON_INFORMATION)
+                          _("Alert"), style=wx.OK|wx.ICON_INFORMATION)
             return
         path = common.app_tree.get_selected_path()
-        print 'path:', path
+        #print 'path:', path
         self._open_app(common.app_tree.app.filename, add_to_history=False)
         common.app_tree.select_path(path)
         
@@ -576,7 +617,7 @@ class wxGladeFrame(wx.Frame):
         infile = misc.FileSelector(_("Open file"),
                                    wildcard="wxGlade files (*.wxg)|*.wxg|"
                                    "wxGlade Template files (*.wgt)|*.wgt|"
-				   "XML files (*.xml)|*.xml|All files|*",
+                                   "XML files (*.xml)|*.xml|All files|*",
                                    flags=wx.OPEN|wx.FILE_MUST_EXIST,
                                    default_path=self.cur_dir)
         if infile:
@@ -628,7 +669,8 @@ class wxGladeFrame(wx.Frame):
             common.app_tree.clear()
             common.property_panel.Reparent(self.frame2)
             common.app_tree.app.saved = True
-            wx.MessageBox(_("Error loading file %s: %s") % (infilename, msg),
+            wx.MessageBox(_("Error loading file %s: %s") % \
+                          (misc.wxstr(infilename), misc.wxstr(msg)),
                           _("Error"), wx.OK|wx.CENTRE|wx.ICON_ERROR)
             # reset the auto-expansion of nodes
             common.app_tree.auto_expand = True
@@ -647,7 +689,8 @@ class wxGladeFrame(wx.Frame):
                             "For more details, look at the full traceback "
                             "on the console.\n"
                             "If you think this is a wxGlade bug,"
-                            " please report it.") % (infilename, msg),
+                            " please report it.") % \
+                          (misc.wxstr(infilename), misc.wxstr(msg)),
                           _("Error"),
                           wx.OK|wx.CENTRE|wx.ICON_ERROR)
             # reset the auto-expansion of nodes
@@ -665,9 +708,8 @@ class wxGladeFrame(wx.Frame):
         common.app_tree.expand()
         if common.app_tree.app.is_template:
             print _("Loaded template")
-            import template
-            common.app_tree.template_data = template.Template(infilename)
-            common.app_tree.filename = None
+            common.app_tree.app.template_data = template.Template(infilename)
+            common.app_tree.app.filename = None
 
         end = time.clock()
         print _('Loading time: %.5f') % (end-start)
@@ -676,14 +718,14 @@ class wxGladeFrame(wx.Frame):
         
         if hasattr(self, 'file_history') and infilename is not None and \
                add_to_history and (not common.app_tree.app.is_template):
-            self.file_history.AddFileToHistory(infilename)
+            self.file_history.AddFileToHistory(misc.wxstr(infilename))
 
         # ALB 2004-10-15
         if config.preferences.autosave and self.autosave_timer is not None:
             self.autosave_timer.Start()
 
         self.user_message(_("Loaded %s (%.2f seconds)") % \
-                          (common.app_tree.app.filename, end-start))
+                          (misc.wxstr(common.app_tree.app.filename), end-start))
 
         return True
 
@@ -696,39 +738,42 @@ class wxGladeFrame(wx.Frame):
         else:
             # check whether we are saving a template
             if os.path.splitext(common.app_tree.app.filename)[1] == ".wgt":
-	        common.app_tree.app.is_template = True
-            try:
-                from cStringIO import StringIO
-                buffer = StringIO()
-                common.app_tree.write(buffer)
-                common.save_file(common.app_tree.app.filename,
-                                 buffer.getvalue(), 'wxg')
-            except (IOError, OSError), msg:
-                common.app_tree.app.saved = False
-                fn = common.app_tree.app.filename
-                wx.MessageBox(_("Error saving app:\n%s") % msg, _("Error"),
-                             wx.OK|wx.CENTRE|wx.ICON_ERROR)
-            except Exception, msg:
-                import traceback; traceback.print_exc()
-                common.app_tree.app.saved = False
-                fn = common.app_tree.app.filename
-                wx.MessageBox(_("An exception occurred while saving file "
-                                "\"%s\".\n"
-                                "This is the error message associated with it:"
-                                "\n        %s\n"
-                                "For more details, look at the full traceback "
-                                "on the console.\nIf you think this is a "
-                                "wxGlade bug,"
-                                " please report it.") % (fn, msg), _("Error"),
-                              wx.OK|wx.CENTRE|wx.ICON_ERROR)
-            else:
-                common.app_tree.app.saved = True
-                common.remove_autosaved() # ALB 2004-10-15
-                # ALB 2004-10-15
-                if config.preferences.autosave and \
-                       self.autosave_timer is not None:
-                    self.autosave_timer.Start()
-                self.user_message(_("Saved %s") % common.app_tree.app.filename)
+                common.app_tree.app.is_template = True
+            self._save_app(common.app_tree.app.filename)
+
+    def _save_app(self, filename):
+        try:
+            from cStringIO import StringIO
+            buffer = StringIO()
+            common.app_tree.write(buffer)
+            common.save_file(filename,
+                             buffer.getvalue(), 'wxg')
+        except (IOError, OSError), msg:
+            common.app_tree.app.saved = False
+            fn = filename
+            wx.MessageBox(_("Error saving app:\n%s") % msg, _("Error"),
+                         wx.OK|wx.CENTRE|wx.ICON_ERROR)
+        except Exception, msg:
+            import traceback; traceback.print_exc()
+            common.app_tree.app.saved = False
+            fn = filename
+            wx.MessageBox(_("An exception occurred while saving file "
+                            "\"%s\".\n"
+                            "This is the error message associated with it:"
+                            "\n        %s\n"
+                            "For more details, look at the full traceback "
+                            "on the console.\nIf you think this is a "
+                            "wxGlade bug,"
+                            " please report it.") % (fn, msg), _("Error"),
+                          wx.OK|wx.CENTRE|wx.ICON_ERROR)
+        else:
+            common.app_tree.app.saved = True
+            common.remove_autosaved() # ALB 2004-10-15
+            # ALB 2004-10-15
+            if config.preferences.autosave and \
+                   self.autosave_timer is not None:
+                self.autosave_timer.Start()
+            self.user_message(_("Saved %s") % filename)
 
     def save_app_as(self, event):
         """\
@@ -750,6 +795,14 @@ class wxGladeFrame(wx.Frame):
             if misc.check_wx_version(2, 3, 3):
                 self.file_history.AddFileToHistory(fn)
 
+    def save_app_as_template(self, event):
+        data = getattr(common.app_tree.app, 'template_data', None)
+        outfile, data = template.save_template(data)
+        if outfile:
+            common.app_tree.app.is_template = True
+            common.app_tree.app.template_data = data
+            self._save_app(outfile)
+
     def cleanup(self, event):
         if self.ask_save():
             # first, let's see if we have to save the geometry...
@@ -768,7 +821,9 @@ class wxGladeFrame(wx.Frame):
                 wx.MessageBox(_('Error saving preferences:\n%s') % e,
                               _('Error'),
                               wx.OK|wx.CENTRE|wx.ICON_ERROR)
-            self._skip_activate = True
+            #self._skip_activate = True
+            self.frame2.Destroy()
+            self.tree_frame.Destroy()
             self.Destroy()
             common.remove_autosaved() # ALB 2004-10-15
             misc.wxCallAfter(wx.GetApp().ExitMainLoop)
@@ -794,11 +849,11 @@ class wxGladeFrame(wx.Frame):
             t.start()
 
     def show_and_raise(self):
-        self.frame2.Show(self.GetMenuBar().IsChecked(self.PROPS_ID))
-        self.tree_frame.Show(self.GetMenuBar().IsChecked(self.TREE_ID))
-        self.Raise()
+        self.frame2.Show()#self.GetMenuBar().IsChecked(self.PROPS_ID))
+        self.tree_frame.Show()#self.GetMenuBar().IsChecked(self.TREE_ID))
         self.frame2.Raise()
         self.tree_frame.Raise()
+        self.Raise()
 
     def hide_all(self):
         self.tree_frame.Hide()
@@ -833,6 +888,17 @@ class wxGladeFrame(wx.Frame):
                               (infile, msg), _("Error"),
                               wx.OK|wx.CENTRE|wx.ICON_ERROR)
 
+    def manage_templates(self, event):
+        to_edit = template.manage_templates()
+        if to_edit is not None and self.ask_save():
+            # edit the template
+            # TODO, you still need to save it manually...
+            self._open_app(to_edit, add_to_history=False)
+            wx.MessageBox(_("To save the changes to the template, edit the "
+                            "GUI as usual,\nand then click "
+                            "File->Save as Template..."), _("Information"),
+                          style=wx.OK|wx.ICON_INFORMATION)
+
 # end of class wxGladeFrame
 
 
@@ -857,17 +923,24 @@ class wxGlade(wx.App):
         wx.ArtProvider.PushProvider(wxGladeArtProvider())
 
         frame = wxGladeFrame()
-        if wx.Platform == '__WXMSW__':
-            def on_activate(event):
-                if event.GetActive() and not frame.IsIconized():
-                    frame.show_and_raise()
-                event.Skip()
-            wx.EVT_ACTIVATE_APP(self, on_activate)
+##         if wx.Platform == '__WXMSW__':
+##             def on_activate(event):
+##                 if event.GetActive() and not frame.IsIconized():
+##                     frame.show_and_raise()
+##                 event.Skip()
+##             wx.EVT_ACTIVATE_APP(self, on_activate)
 
         self.SetTopWindow(frame)
         self.SetExitOnFrameDelete(True)
+
+        wx.EVT_IDLE(self, self.on_idle)
+        
         return True
 
+    def on_idle(self, event):
+        common.message.flush()
+        event.Skip()
+
 # end of class wxGlade
 
 
@@ -876,7 +949,6 @@ def main(filename=None):
     if filename is not None, loads it
     """
     # first thing to do, patch wxSizerPtr's Insert if needed...
-##     from wxPython import wx
 ##     if wx.__version__ == '2.4.0.2':
 ##         wxSizerPtr.Insert = misc.sizer_fixed_Insert
 
