@@ -1,15 +1,18 @@
 # tree.py: classes to handle and display the structure of a wxGlade app
-# $Id: tree.py,v 1.55 2007/03/27 07:02:07 agriggio Exp $
+# $Id: tree.py,v 1.56 2007/08/07 12:13:44 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
-#from wxPython.wx import *
 import wx
 from xml.sax.saxutils import quoteattr
 import misc, common, os.path
 
+try: set
+except NameError: from sets import Set as set
+
+
 class Tree:
     """\
     A class to represent a hierarchy of widgets.
@@ -46,7 +49,7 @@ class Tree:
             try: return self.widget.name
             except AttributeError: return repr(self.widget)
 
-        def write(self, outfile, tabs):
+        def write(self, outfile, tabs, class_names=None):
             """\
             Writes the xml code for the widget to the given output file
             """
@@ -63,8 +66,28 @@ class Tree:
             fwrite('    ' * tabs + '<object class=%s name=%s base=%s%s>\n'
                    % (quoteattr(w.klass), quoteattr(w.name),
                       quoteattr(classname), no_custom))
+            
+            def write_by_omitter(widget, name, file, tabs):
+                """
+                Write to given output file:
+                - value of property
+                - values of properties that previous can block
+                - any omitter can be blocked as well
+                """
+                widget.properties[name].write(file, tabs)
+                if getattr(widget, 'get_property_blocking', None):
+                    items = widget.get_property_blocking(name)
+                    if items:
+                        for name in items:
+                            write_by_omitter(widget, name, file, tabs)
+                                            
             for p in w.properties:
-                w.properties[p].write(outfile, tabs+1)
+                if not getattr(w.properties[p], '_omitter', None):
+                    write_by_omitter(w, p, outfile, tabs+1)
+
+            if class_names is not None and \
+                   w.__class__.__name__ != 'CustomWidget':
+                class_names.add(w.klass)
 
             if isinstance(w, edit_sizers.SizerBase):
                 maxpos = len(w.children)
@@ -86,12 +109,12 @@ class Tree:
                                '<object class="sizeritem">\n')
                         sp = c.widget.sizer_properties
                         for p in sp: sp[p].write(outfile, tabs+2)
-                        c.write(outfile, tabs+2)
+                        c.write(outfile, tabs+2, class_names)
                         fwrite('    ' * (tabs+1) + '</object>\n')
                     else:
                         c.write(outfile, tabs+1)
             elif self.children is not None:
-                for c in self.children: c.write(outfile, tabs+1)
+                for c in self.children: c.write(outfile, tabs+1, class_names)
             fwrite('    ' * tabs + '</object>\n')
 
     # end of class Node
@@ -105,7 +128,7 @@ class Tree:
                         # itself a dictionary, one for each toplevel widget...
 
     def _find_toplevel(self, node):
-        assert node is not None, "None node in _find_toplevel"
+        assert node is not None, _("None node in _find_toplevel")
         if node.parent is self.root:
             return node.parent
         while node.parent is not self.root:
@@ -194,8 +217,16 @@ class Tree:
         language = self.app.get_language()
         encoding = self.app.get_encoding()
         use_gettext = str(int(self.app.use_gettext))
-	is_template = str(int(self.app.is_template))
+        is_template = str(int(self.app.is_template))
         overwrite = str(int(self.app.overwrite))
+        indent_amount = str(int(self.app.indent_amount))
+        indent_symbol = int(self.app.indent_mode)
+        if indent_symbol == 0:
+            indent_symbol = "tab"
+        else:
+            indent_symbol = "space"
+        source_ext = '.' + self.app.source_ext
+        header_ext = '.' + self.app.header_ext
         # ALB 2004-01-18
         #use_new_namespace = str(int(self.app.get_use_new_namespace()))
         use_new_namespace = str(int(not self.app.get_use_old_namespace()))
@@ -204,16 +235,23 @@ class Tree:
         outfile.write('<application path=%s name=%s class=%s option=%s ' \
                       'language=%s top_window=%s encoding=%s ' \
                       'use_gettext=%s overwrite=%s '
-                      'use_new_namespace=%s for_version=%s is_template=%s>\n' \
-                      % tuple(map(quoteattr,
-                                  [outpath, name, klass, option, language,
-                                   top_window, encoding, use_gettext,
-                                   overwrite, use_new_namespace, for_version,is_template]))
-                      )
+                      'use_new_namespace=%s for_version=%s is_template=%s '\
+                      'indent_amount=%s indent_symbol=%s '\
+                      'source_extension=%s header_extension=%s>\n' \
+                      % tuple([quoteattr(common._encode_to_xml(i)) for i in 
+                               (outpath, name, klass, option, language,
+                                top_window, encoding, use_gettext,
+                                overwrite, use_new_namespace, for_version,
+                                is_template, indent_amount, indent_symbol,
+                                source_ext, header_ext)]))
+        if self.app.is_template and getattr(self.app, 'template_data', None):
+            self.app.template_data.write(outfile, tabs+1)
+        class_names = set()
         if self.root.children is not None:
             for c in self.root.children:
-                c.write(outfile, tabs+1)
+                c.write(outfile, tabs+1, class_names)
         outfile.write('</application>\n')
+        return class_names
 
     def change_node(self, node, widget):
         """\
@@ -261,8 +299,10 @@ class WidgetTree(wx.TreeCtrl, Tree):
                                              'icons/application.xpm'),
                                 wx.BITMAP_TYPE_XPM))
         for w in WidgetTree.images:
-            WidgetTree.images[w] = image_list.Add(wx.Bitmap(
-                WidgetTree.images[w], wx.BITMAP_TYPE_XPM))
+##             WidgetTree.images[w] = image_list.Add(wx.Bitmap(
+##                 WidgetTree.images[w], wx.BITMAP_TYPE_XPM))
+            WidgetTree.images[w] = image_list.Add(
+                misc.get_xpm_bitmap(WidgetTree.images[w]))
         self.AssignImageList(image_list)
         root_node.item = self.AddRoot(_('Application'), 0)
         self.SetPyData(root_node.item, root_node)
@@ -316,6 +356,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
             self.Expand(parent.item)
             self.select_item(child)
         child.widget.show_properties()
+        self.app.check_codegen(child.widget)
 
     def insert(self, child, parent, pos, image=None):
         """\
@@ -349,6 +390,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
             self.Expand(parent.item)
             self.select_item(child)
         child.widget.show_properties()
+        self.app.check_codegen(child.widget)
 
     def remove(self, node=None):
         self.app.saved = False # update the status of the app
@@ -579,7 +621,9 @@ class WidgetTree(wx.TreeCtrl, Tree):
         from edit_sizers import SizerBase
         ret = []
         w = self.cur_widget
+        oldw = None
         while w is not None:
+            oldw = w
             ret.append(w.name)
             sizer = getattr(w, 'sizer', None)
             if getattr(w, 'parent', "") is None:
@@ -592,6 +636,12 @@ class WidgetTree(wx.TreeCtrl, Tree):
                 else:
                     w = w.parent
         ret.reverse()
+        # ALB 2007-08-28: remember also the position of the toplevel window in
+        # the selected path
+        if oldw is not None:
+            assert oldw.widget
+            pos = misc.get_toplevel_parent(oldw.widget).GetPosition()
+            ret[0] = (ret[0], pos)
         return ret
 
     def select_path(self, path):
@@ -603,9 +653,13 @@ class WidgetTree(wx.TreeCtrl, Tree):
         item, cookie = self._get_first_child(self.GetRootItem())
         itemok = None
         parent = None
+        pos = None
         while item.Ok() and index < len(path):
             widget = self.GetPyData(item).widget
-            if misc.streq(widget.name, path[index]):
+            name = path[index]
+            if index == 0 and type(name) == type(()):
+                name, pos = name
+            if misc.streq(widget.name, name):
                 #print 'OK:', widget.name
                 #self.EnsureVisible(item)
                 itemok = item
@@ -621,6 +675,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
             node = self.GetPyData(itemok)
             if parent is not None:
                 self.show_widget(parent, True)
+                if pos is not None:
+                    misc.get_toplevel_parent(parent.widget).SetPosition(pos)
             self.select_item(node)
         
     def _get_first_child(self, item):
