@@ -1,5 +1,5 @@
 # panel.py: wxPanel objects
-# $Id: panel.py,v 1.36 2007/03/27 07:01:56 agriggio Exp $
+# $Id: panel.py,v 1.38 2007/08/07 12:15:21 agriggio Exp $
 #
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
@@ -13,6 +13,9 @@ from edit_windows import ManagedBase, TopLevelBase
 
 
 class PanelBase(object):
+
+    _custom_base_classes = True
+
     def __init__(self, style=wx.TAB_TRAVERSAL):
         """\
         Class to handle wxPanel objects
@@ -25,7 +28,7 @@ class PanelBase(object):
                                                     self.set_no_custom_class)
         self.properties['no_custom_class'] = CheckBoxProperty(
             self, 'no_custom_class',
-            label="Don't generate code for this custom class")
+            label=_("Don't generate code for this custom class"))
         # ------
         self.style = style
         self.access_functions['style'] = (self.get_style, self.set_style)
@@ -36,7 +39,8 @@ class PanelBase(object):
                            wx.NO_FULL_REPAINT_ON_RESIZE,
                            wx.FULL_REPAINT_ON_RESIZE,
                            wx.CLIP_CHILDREN]
-        style_labels = ('#section#Style', 'wxSIMPLE_BORDER', 'wxDOUBLE_BORDER',
+        style_labels = ('#section#' + _('Style'),
+                        'wxSIMPLE_BORDER', 'wxDOUBLE_BORDER',
                         'wxSUNKEN_BORDER', 'wxRAISED_BORDER',
                         'wxSTATIC_BORDER',
                         'wxNO_BORDER', 'wxNO_3D', 'wxTAB_TRAVERSAL',
@@ -49,12 +53,12 @@ class PanelBase(object):
                                                self.set_scrollable)
         self.scrollable = False
         self.properties['scrollable'] = CheckBoxProperty(
-            self, 'scrollable', None)
+            self, 'scrollable', None, label=_("scrollable"))
         self.scroll_rate = (10, 10)
         self.access_functions['scroll_rate'] = (self.get_scroll_rate,
                                                 self.set_scroll_rate)
-        self.properties['scroll_rate'] = TextProperty(self, 'scroll_rate',
-                                                      None, can_disable=True)
+        self.properties['scroll_rate'] = TextProperty(
+            self, 'scroll_rate', None, can_disable=True, label=_("scroll_rate"))
 
     def finish_widget_creation(self):
         super(PanelBase, self).finish_widget_creation(
@@ -66,8 +70,21 @@ class PanelBase(object):
         # this must be done here since ManagedBase.finish_widget_creation
         # normally sets EVT_LEFT_DOWN to update_wiew
         if not self.widget.Disconnect(-1, -1, wx.wxEVT_LEFT_DOWN):
-            print "EditPanel: Unable to disconnect the event hanlder"
+            print _("EditPanel: Unable to disconnect the event hanlder")
         wx.EVT_LEFT_DOWN(self.widget, self.drop_sizer)
+        #wx.EVT_SCROLLWIN(self.widget, self._update_markers)
+
+    def _update_markers(self, event):
+        def get_pos():
+            x, y = self.widget.GetPosition()
+            xx, yy = self.widget.GetViewStart()
+            return x+xx, y+yy
+        old = self.widget.GetPosition
+        self.widget.GetPosition = get_pos
+        #print self.widget, self.sel_marker.owner
+        self.sel_marker.update()
+        self.widget.GetPosition = old
+        event.Skip()
 
     def create_properties(self):
         super(PanelBase, self).create_properties()
@@ -78,9 +95,9 @@ class PanelBase(object):
         szr.Add(self.properties['no_custom_class'].panel, 0, wx.EXPAND)
         label = self.properties['no_custom_class'].cb
         label.SetToolTip(
-            wx.ToolTip('If this is a custom class, setting this property '
+            wx.ToolTip(_('If this is a custom class, setting this property '
                        'prevents wxGlade\nfrom generating the class definition'
-                       ' code'))
+                       ' code')))
         self.properties['style'].display(panel)
         szr.Add(self.properties['style'].panel, 0, wx.EXPAND)
         self.properties['scrollable'].display(panel)
@@ -104,7 +121,8 @@ class PanelBase(object):
             self.widget.SetAutoLayout(True)
             self.widget.SetSizer(self.top_sizer.widget)
             self.widget.Layout()
-            #self.sizer.set_item(self.pos, size=self.widget.GetBestSize())
+        elif self.top_sizer is None and self.widget:
+            self.widget.SetSizer(None)
 
     def drop_sizer(self, event):
         if self.top_sizer or not common.adding_sizer:
@@ -118,10 +136,8 @@ class PanelBase(object):
 
     def get_widget_best_size(self):
         if self.top_sizer and self.widget.GetSizer():
-            #return self.widget.GetSizer().CalcMin()
             self.top_sizer.fit_parent()
             return self.widget.GetSize()
-        #return wx.Panel.GetBestSize(self.widget)
         return wx.ScrolledWindow.GetBestSize(self.widget)
 
     def get_style(self):
@@ -140,7 +156,6 @@ class PanelBase(object):
         for v in range(len(value)):
             if value[v]:
                 self.style |= self.style_pos[v]
-        #if self.widget: self.widget.SetWindowStyleFlag(self.style)
 
     def get_scrollable(self):
         return self.scrollable
@@ -233,12 +248,12 @@ class EditPanel(PanelBase, ManagedBase):
             if not self._rmenu:
                 COPY_ID, REMOVE_ID, CUT_ID = [wx.NewId() for i in range(3)]
                 self._rmenu = misc.wxGladePopupMenu(self.name)
-                misc.append_item(self._rmenu, REMOVE_ID, 'Remove\tDel',
-                                 'remove.xpm')
-                misc.append_item(self._rmenu, COPY_ID, 'Copy\tCtrl+C',
-                                 'copy.xpm')
-                misc.append_item(self._rmenu, CUT_ID, 'Cut\tCtrl+X',
-                                 'cut.xpm')
+                misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
+                                 wx.ART_DELETE)
+                misc.append_item(self._rmenu, COPY_ID, _('Copy\tCtrl+C'),
+                                 wx.ART_COPY)
+                misc.append_item(self._rmenu, CUT_ID, _('Cut\tCtrl+X'),
+                                 wx.ART_CUT)
                 def bind(method):
                     return lambda e: misc.wxCallAfter(method)
                 wx.EVT_MENU(self.widget, REMOVE_ID, bind(self.remove))
@@ -246,10 +261,15 @@ class EditPanel(PanelBase, ManagedBase):
                 wx.EVT_MENU(self.widget, CUT_ID, bind(self.clipboard_cut))
                 # paste
                 PASTE_ID = wx.NewId()
-                misc.append_item(self._rmenu, PASTE_ID, 'Paste\tCtrl+V',
-                                 'paste.xpm')
+                misc.append_item(self._rmenu, PASTE_ID, _('Paste\tCtrl+V'),
+                                 wx.ART_PASTE)
                 wx.EVT_MENU(self.widget, PASTE_ID, bind(self.clipboard_paste))
-                
+                PREVIEW_ID = wx.NewId()
+                self._rmenu.AppendSeparator()
+                misc.append_item(self._rmenu, PREVIEW_ID, _('Preview'))
+                wx.EVT_MENU(self.widget, PREVIEW_ID, bind(self.preview_parent))
+
+            self.setup_preview_menu()
             self.widget.PopupMenu(self._rmenu, event.GetPosition())
 
     def clipboard_paste(self, *args):
@@ -260,7 +280,7 @@ class EditPanel(PanelBase, ManagedBase):
                 common.app_tree.app.saved = False
                 self.widget.SetSize(size)
         except xml_parse.XmlParsingError, e:
-            print '\nwxGlade-WARNING: only sizers can be pasted here'
+            print _('\nwxGlade-WARNING: only sizers can be pasted here')
             
 # end of class EditPanel
 
@@ -280,6 +300,11 @@ class EditTopLevelPanel(PanelBase, TopLevelBase):
     def create_widget(self):
         win = wx.Frame(common.palette, -1, misc.design_title(self.name),
                        size=(400, 300)) 
+        import os
+        icon = wx.EmptyIcon()
+        xpm = os.path.join(common.wxglade_path, 'icons', 'panel.xpm')
+        icon.CopyFromBitmap(misc.get_xpm_bitmap(xpm))
+        win.SetIcon(icon)
         #self.widget = wx.Panel(win, self.id, style=0)
         self.widget = wx.ScrolledWindow(win, self.id, style=0)
         wx.EVT_ENTER_WINDOW(self.widget, self.on_enter)
@@ -315,16 +340,13 @@ class EditTopLevelPanel(PanelBase, TopLevelBase):
 
     def on_size(self, event):
         w, h = event.GetSize()
-##         if (wx.Platform == '__WXMSW__' or wx.Platform == '__WXMAC__') \
-##                and self.skip_on_size:
         if self.skip_on_size:
             self.skip_on_size = False
             return
         super(EditTopLevelPanel, self).on_size(event)
-        #w, h = self.widget.GetClientSize()
         self.skip_on_size = True
-        #self.widget.GetParent().SetClientSize((w+2, h+2))
-        self.widget.GetParent().SetClientSize((w+2, h+2))
+        if self.widget.GetParent().GetClientSize() != (w, h):
+            self.widget.GetParent().SetClientSize((w+2, h+2))
 
     def set_scrollable(self, value):
         super(EditTopLevelPanel, self).set_scrollable(value)
@@ -367,9 +389,9 @@ def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     """
     from xml_parse import XmlParsingError
     try: name = attrs['name']
-    except KeyError: raise XmlParsingError, "'name' attribute missing"
+    except KeyError: raise XmlParsingError, _("'name' attribute missing")
     if not sizer or not sizeritem:
-        raise XmlParsingError, "sizer or sizeritem object cannot be None"
+        raise XmlParsingError, _("sizer or sizeritem object cannot be None")
     panel = EditPanel(name, parent, wx.NewId(), sizer, pos,
                       common.property_panel, True, style=0)
     sizer.set_item(panel.pos, option=sizeritem.option, flag=sizeritem.flag,
@@ -384,7 +406,7 @@ def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
 def xml_toplevel_builder(attrs, parent, sizer, sizeritem, pos=None):
     from xml_parse import XmlParsingError
     try: label = attrs['name']
-    except KeyError: raise XmlParsingError, "'name' attribute missing"
+    except KeyError: raise XmlParsingError, _("'name' attribute missing")
     panel = EditTopLevelPanel(label, parent, wx.NewId(), common.property_panel,
                               show=False, style=0)
     node = Tree.Node(panel)
